{ **************************************************************************** }
{ *                                                                          * }
{ * TRON (Tomytronic) Simulator                                              * }
{ * by Luca "MADrigal" Antignano, May 2001 (revision: August 2021)           * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit gfxinit;
{ this unit contains the graphics initialization (skin-based system)           }

interface

uses
  Forms, Controls, StdCtrls, SysUtils, Classes, Graphics, ExtCtrls, Dialogs,
  Jpeg, Pngimage, IniFiles;

{ * PROCEDURES: SKIN INITIALIZATION                                          * }
procedure Gfx_Game_Init;
procedure Gfx_Manual_Init;

const
  INIFILE = 'Config.ini';

{}IMAGES = 207;

{}DEFAULTHEIGHT    = 700;
{}DEFAULTWIDTH     = 766;
  DEFAULTSKINDIR   = 'Graphics\Skin\';
  DEFAULTWINMODE   = 'Shaped';

  DEFAULTTRCOLOR   = '$00ff00ff';

  DEFAULTBDDIR     = 'Graphics\Backdrop\';
  DEFAULTBDFILE    = 'nil';
  DEFAULTBDSTRETCH = 0;
{}DEFAULTBGCOLOR   = '$00000000';
  DEFAULTOFFSETX   = 'Center';
  DEFAULTOFFSETY   = 'Center';
  DEFAULTLOGOFILE  = 'nil';
  DEFAULTLOGOPOS   = 1;

  DEFAULTMANUALDIR = 'Graphics\Manual\';
{}DEFAULTMANHEIGHT = 785;
{}DEFAULTMANWIDTH  = 580;

var
  IF_File: TIniFile;
  aImages: array [1..IMAGES] of TImage;
{}//sVideoMode: string;

implementation

uses
  manual, splash, unit1;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: SKIN INITIALIZATION                                          * }
{ *                                                                          * }
{ **************************************************************************** }

procedure Gfx_Game_Init;
{ initializes the graphics on the main game form                               }
  var
    // service variables
    i,
    iOffsetX,
    iOffsetY: integer;

    // temporary variables (used to read/write from 'ini file')
    sSkinDir,
    sBackdropDir,
    sBackdropFile,
    sGameOffsetX,
    sGameOffsetY,
{}  sLogoFile: string;
{}//sWindowMode

    // image variables
    sFilename: string;
begin
  // initialises array of images
  with Form1 do
    begin
      aImages[1]   := im_background;             aImages[2]   := im_background_screen_top;
                                                 aImages[3]   := im_background_screen_on;

      aImages[4]   := btn_power_on;              aImages[5]   := btn_power_top;
      aImages[6]   := btn_joystick_down;         aImages[7]   := btn_joystick_top;
      aImages[206] := btn_action_down;           aImages[207] := btn_action_top;

      aImages[8]  := btn_ctrlr_overlay_up;
      aImages[9]  := btn_ctrlr_overlay_left;
      aImages[10] := btn_ctrlr_overlay_right;
      aImages[11] := btn_ctrlr_overlay_down;

      aImages[12] := im_panel_digit_pos_1; aImages[13] := im_panel_digit_pos_2; aImages[14] := im_panel_digit_pos_3; aImages[15] := im_panel_digit_pos_4;

      aImages[16] := im_panel_number_0; aImages[17] := im_panel_number_1; aImages[18] := im_panel_number_2; aImages[19] := im_panel_number_3; aImages[20] := im_panel_number_4;
      aImages[21] := im_panel_number_5; aImages[22] := im_panel_number_6; aImages[23] := im_panel_number_7; aImages[24] := im_panel_number_8; aImages[25] := im_panel_number_9;

      aImages[26] := im_tron_body_5; aImages[27] := im_tron_hand_5; aImages[28] := im_tron_disc_5;
      aImages[29] := im_tron_body_4; aImages[30] := im_tron_hand_4; aImages[31] := im_tron_disc_4;
      aImages[32] := im_tron_body_3; aImages[33] := im_tron_hand_3; aImages[34] := im_tron_disc_3;
      aImages[35] := im_tron_body_2; aImages[36] := im_tron_hand_2; aImages[37] := im_tron_disc_2;
      aImages[38] := im_tron_body_1; aImages[39] := im_tron_hand_1; aImages[40] := im_tron_disc_1;

      aImages[41] := im_sark_body_4; aImages[42] := im_sark_hand_4; aImages[43] := im_sark_disc_4;
      aImages[44] := im_sark_body_3; aImages[45] := im_sark_hand_3; aImages[46] := im_sark_disc_3;
      aImages[47] := im_sark_body_2; aImages[48] := im_sark_hand_2; aImages[49] := im_sark_disc_2;
      aImages[50] := im_sark_body_1; aImages[51] := im_sark_hand_1; aImages[52] := im_sark_disc_1;

      aImages[53] := im_mcp_cover_1; aImages[54] := im_mcp_cover_2; aImages[55] := im_mcp_cover_3; aImages[56] := im_mcp_cover_4;
      aImages[57] := im_mcp_body;
      aImages[58] := im_mcp_face_1; aImages[59] := im_mcp_face_2; aImages[60] := im_mcp_face_3; aImages[61] := im_mcp_face_4; aImages[62] := im_mcp_face_5;

      aImages[63] := im_cycle_01_05h; aImages[64] := im_cycle_02_05h; aImages[65] := im_cycle_03_05h; aImages[66] := im_cycle_04_05h; aImages[67] := im_cycle_05_05h; aImages[68] := im_cycle_06_05h; aImages[69] := im_cycle_07_05h;
                                      aImages[70] := im_cycle_01_04h; aImages[71] := im_cycle_02_04h; aImages[72] := im_cycle_03_04h; aImages[73] := im_cycle_04_04h; aImages[74] := im_cycle_05_04h; aImages[75] := im_cycle_06_04h; aImages[76] := im_cycle_07_04h;
                                      aImages[77] := im_cycle_01_03h; aImages[78] := im_cycle_02_03h; aImages[79] := im_cycle_03_03h; aImages[80] := im_cycle_04_03h; aImages[81] := im_cycle_05_03h; aImages[82] := im_cycle_06_03h; aImages[83] := im_cycle_07_03h;
                                      aImages[84] := im_cycle_01_02h; aImages[85] := im_cycle_02_02h; aImages[86] := im_cycle_03_02h; aImages[87] := im_cycle_04_02h; aImages[88] := im_cycle_05_02h; aImages[89] := im_cycle_06_02h; aImages[90] := im_cycle_07_02h;
                                      aImages[91] := im_cycle_00_01h; aImages[92] := im_cycle_01_01h; aImages[93] := im_cycle_02_01h; aImages[94] := im_cycle_03_01h; aImages[95] := im_cycle_04_01h; aImages[96] := im_cycle_05_01h; aImages[97] := im_cycle_06_01h; aImages[98] := im_cycle_07_01h; aImages[99] := im_cycle_08_01h;

      aImages[100] := im_cycle_01_05v; aImages[101] := im_cycle_02_05v; aImages[102] := im_cycle_03_05v; aImages[103] := im_cycle_04_05v; aImages[104] := im_cycle_05_05v; aImages[105] := im_cycle_06_05v; aImages[106] := im_cycle_07_05v;
                                       aImages[107] := im_cycle_01_04v; aImages[108] := im_cycle_02_04v; aImages[109] := im_cycle_03_04v; aImages[110] := im_cycle_04_04v; aImages[111] := im_cycle_05_04v; aImages[112] := im_cycle_06_04v; aImages[113] := im_cycle_07_04v;
                                       aImages[114] := im_cycle_01_03v; aImages[115] := im_cycle_02_03v; aImages[116] := im_cycle_03_03v; aImages[117] := im_cycle_04_03v; aImages[118] := im_cycle_05_03v; aImages[119] := im_cycle_06_03v; aImages[120] := im_cycle_07_03v;
                                       aImages[121] := im_cycle_01_02v; aImages[122] := im_cycle_02_02v; aImages[123] := im_cycle_03_02v; aImages[124] := im_cycle_04_02v; aImages[125] := im_cycle_05_02v; aImages[126] := im_cycle_06_02v; aImages[127] := im_cycle_07_02v;
                                       aImages[128] := im_cycle_01_01v; aImages[129] := im_cycle_02_01v; aImages[130] := im_cycle_03_01v; aImages[131] := im_cycle_04_01v; aImages[132] := im_cycle_05_01v; aImages[133] := im_cycle_06_01v; aImages[134] := im_cycle_07_01v;

      aImages[135] := im_disc_01_05h; aImages[136] := im_disc_02_05h; aImages[137] := im_disc_03_05h; aImages[138] := im_disc_04_05h; aImages[139] := im_disc_05_05h; aImages[140] := im_disc_06_05h; aImages[141] := im_disc_07_05h;
                                      aImages[142] := im_disc_01_04h; aImages[143] := im_disc_02_04h; aImages[144] := im_disc_03_04h; aImages[145] := im_disc_04_04h; aImages[146] := im_disc_05_04h; aImages[147] := im_disc_06_04h; aImages[148] := im_disc_07_04h;
                                      aImages[149] := im_disc_01_03h; aImages[150] := im_disc_02_03h; aImages[151] := im_disc_03_03h; aImages[152] := im_disc_04_03h; aImages[153] := im_disc_05_03h; aImages[154] := im_disc_06_03h; aImages[155] := im_disc_07_03h;
                                      aImages[156] := im_disc_01_02h; aImages[157] := im_disc_02_02h; aImages[158] := im_disc_03_02h; aImages[159] := im_disc_04_02h; aImages[160] := im_disc_05_02h; aImages[161] := im_disc_06_02h; aImages[162] := im_disc_07_02h; aImages[163] := im_disc_08_02h;
                                      aImages[164] := im_disc_01_01h; aImages[165] := im_disc_02_01h; aImages[166] := im_disc_03_01h; aImages[167] := im_disc_04_01h; aImages[168] := im_disc_05_01h; aImages[169] := im_disc_06_01h; aImages[170] := im_disc_07_01h;

      aImages[171] := im_disc_01_05v; aImages[172] := im_disc_02_05v; aImages[173] := im_disc_03_05v; aImages[174] := im_disc_04_05v; aImages[175] := im_disc_05_05v; aImages[176] := im_disc_06_05v; aImages[177] := im_disc_07_05v;
                                      aImages[178] := im_disc_01_04v; aImages[179] := im_disc_02_04v; aImages[180] := im_disc_03_04v; aImages[181] := im_disc_04_04v; aImages[182] := im_disc_05_04v; aImages[183] := im_disc_06_04v; aImages[184] := im_disc_07_04v;
                                      aImages[185] := im_disc_01_03v; aImages[186] := im_disc_02_03v; aImages[187] := im_disc_03_03v; aImages[188] := im_disc_04_03v; aImages[189] := im_disc_05_03v; aImages[190] := im_disc_06_03v; aImages[191] := im_disc_07_03v;
                                      aImages[192] := im_disc_01_02v; aImages[193] := im_disc_02_02v; aImages[194] := im_disc_03_02v; aImages[195] := im_disc_04_02v; aImages[196] := im_disc_05_02v; aImages[197] := im_disc_06_02v; aImages[198] := im_disc_07_02v;
                                      aImages[199] := im_disc_01_01v; aImages[200] := im_disc_02_01v; aImages[201] := im_disc_03_01v; aImages[202] := im_disc_04_01v; aImages[203] := im_disc_05_01v; aImages[204] := im_disc_06_01v; aImages[205] := im_disc_07_01v;
    end;

  // TO BE USED TO GENERATE THE SKIN FILE ONLY (TO BE COMMENTED THEREAFTER)
{  IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'Export.ini');

  for i := 1 to IMAGES do
    begin
      if aImages[i].Picture.Graphic = nil then
        sFilename := 'nil'
      else
        if aImages[i].Picture.Graphic is TJPEGImage then
          sFilename := aImages[i].Name + '.jpg'
        else
          if aImages[i].Picture.Graphic is TBitmap then
            sFilename := aImages[i].Name + '.bmp'
          else
            if aImages[i].Picture.Graphic is TPNGObject then
              sFilename := aImages[i].Name + '.png';

      IF_file.WriteString('Image' + IntToStr(i),  'Filename', sFilename);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Height',   aImages[i].Height);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Left',     aImages[i].Left);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Top',      aImages[i].Top);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Width',    aImages[i].Width);
    end;

  IF_File.Destroy; }

  // loads configuration settings from the Config.ini file
  if not FileExists(ExtractFilePath(ParamStr(0)) + INIFILE) then
    begin
      MessageDlg('Configuration file (Config.ini) not found. Please reinstall the game.', mtError, [mbOK], 0);
      Application.Terminate;
    end
  else
    begin
      // reads init values from 'ini file' -- with defaults if no 'ini file' found
      IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);

      // game form initialization (game graphics and backdrop)
      with Form1 do
        begin
          sSkinDir     := IncludeTrailingPathDelimiter(IF_file.ReadString('Common', 'SkinDir', DEFAULTSKINDIR));
          ClientHeight := IF_file.ReadInteger('Common', 'GameHeight', DEFAULTHEIGHT);
          ClientWidth  := IF_file.ReadInteger('Common', 'GameWidth',  DEFAULTWIDTH);

          // use Fullscreen or Shaped mode
{}        if bFullscreenMode then
            begin                                                               // Fullscreen
              // sets backdrop image and background colour
              sBackdropDir  := IncludeTrailingPathDelimiter(IF_file.ReadString('Fullscreen', 'BackdropDir',  DEFAULTBDDIR));
              sBackdropFile := IF_file.ReadString('Fullscreen', 'BackdropFile', DEFAULTBDFILE);

              if (sBackdropFile <> '')
                and (sBackdropFile <> 'nil') then
                if FileExists(sBackdropDir + sBackdropFile) then
                  begin
                    im_backdrop.Picture.LoadFromFile(sBackdropDir + sBackdropFile);
                    im_backdrop.Stretch := Boolean(IF_file.ReadInteger('Fullscreen', 'BackdropStretch', DEFAULTBDSTRETCH));
{}                  im_backdrop.Visible := true;
                  end;

              Color := StrToInt(IF_file.ReadString('Fullscreen', 'BgColor', DEFAULTBGCOLOR));
              TransparentColor := false;

              // sets new coordinates for the handheld game graphics
              sGameOffsetX := IF_file.ReadString('Fullscreen', 'GameOffsetX', DEFAULTOFFSETX);
              sGameOffsetY := IF_file.ReadString('Fullscreen', 'GameOffsetY', DEFAULTOFFSETY);

              if StrToIntDef(sGameOffsetX, 0) <> 0 then iOffsetX := StrToInt(sGameOffsetX) else iOffsetX := Trunc((Screen.Width  - ClientWidth)  / 2);
              if StrToIntDef(sGameOffsetY, 0) <> 0 then iOffsetY := StrToInt(sGameOffsetY) else iOffsetY := Trunc((Screen.Height - ClientHeight) / 2);

              // sets the form as full screen
              ClientWidth  := Screen.Width;
              ClientHeight := Screen.Height;

              // sets logo (watermark) image and its position
              sLogoFile := IF_file.ReadString('Fullscreen', 'LogoFile', DEFAULTLOGOFILE);

              if (sLogoFile <> '')
                and (sLogoFile <> 'nil') then
                if FileExists(sBackdropDir + sLogoFile) then
                  with Form1 do
                    begin
                      im_backdrop_logo.Picture.LoadFromFile(sBackdropDir + sLogoFile);
                      im_backdrop_logo.AutoSize := true;
{}                    im_backdrop_logo.Visible := true;

                      case IF_file.ReadInteger('Fullscreen', 'LogoPosition', DEFAULTLOGOPOS) of
                        0: begin im_backdrop_logo.Left := 0;                                    im_backdrop_logo.Top := 0; end; // top-left
                        1: begin im_backdrop_logo.Left := ClientWidth - im_backdrop_logo.Width; im_backdrop_logo.Top := 0; end; // top-right
                        2: begin im_backdrop_logo.Left := 0;                                    im_backdrop_logo.Top := ClientHeight - im_backdrop_logo.Height; end; // bottom-left
                        3: begin im_backdrop_logo.Left := ClientWidth - im_backdrop_logo.Width; im_backdrop_logo.Top := ClientHeight - im_backdrop_logo.Height; end; // bottom-right
                      end; // case..of
                    end;
            end
          else                                                                  // Shaped
            begin
              // sets coordinates for the handheld game graphics
              iOffsetX := 0;
              iOffsetY := 0;

              // sets the form as 'shaped'
              Color                 := StrToInt(IF_file.ReadString('VideoMode', 'TranspColor', DEFAULTTRCOLOR));
              TransparentColorValue := StrToInt(IF_file.ReadString('VideoMode', 'TranspColor', DEFAULTTRCOLOR));
              TransparentColor      := true;

              // disables the backdrop
              im_backdrop.Visible := false;
              im_backdrop_logo.Visible := false;
            end;
        end;

      // game skin initialization
      for i := 1 to IMAGES do
        begin
          sFilename := IF_file.ReadString('Image' + IntToStr(i), 'Filename', '');

          if (sFilename <> '')
            and (sFilename <> 'nil') then
            if FileExists(sSkinDir + sFilename) then
              // loads picture into the image
              aImages[i].Picture.LoadFromFile(sSkinDir + sFilename)
            else
              // image expected but not found -> creates a coloured square as a placeholder
              begin
                aImages[i].Picture.Bitmap.Create;
                aImages[i].Picture.Bitmap.Width  := 10;
                aImages[i].Picture.Bitmap.Height := 10;
                aImages[i].Picture.Bitmap.Canvas.Create;
                aImages[i].Picture.Bitmap.Canvas.Pen.Color := 0;
                aImages[i].Picture.Bitmap.Canvas.Pen.Style := psSolid;
                aImages[i].Picture.Bitmap.Canvas.Brush.Color := $000000ff;
                aImages[i].Picture.Bitmap.Canvas.Rectangle(0, 0, 10, 10);
              end;

          aImages[i].Height := IF_file.ReadInteger('Image' + IntToStr(i), 'Height', 0);
          aImages[i].Left   := IF_file.ReadInteger('Image' + IntToStr(i), 'Left',   0) + iOffsetX;
          aImages[i].Top    := IF_file.ReadInteger('Image' + IntToStr(i), 'Top',    0) + iOffsetY;
          aImages[i].Width  := IF_file.ReadInteger('Image' + IntToStr(i), 'Width',  0);
        end;

      // close Inifile
      IF_File.Destroy;
    end;

  // initializes the on-screen game menu
  with Form1 do
    begin
{}    btn_game_menu_top.Visible := bFullscreenMode;
{}    btn_game_menu_background.Visible := bFullscreenMode;

      btn_game_menu_background.Height := 32;
      btn_game_menu_background.Width := 36;
    end;
end;





procedure Gfx_Manual_Init;
{ initializes the graphics of the instructions manual form                     }
  var
    // service variables
    i: integer;

    // temporary variables (used to read/write from 'ini file')
    sManualDir: string;

    // image variables
    sFilename: string;
begin
  // reads init values from 'ini file' -- with defaults if no 'ini file' found
  IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);

  // manual form initialization (graphics and sizes)
  with ManualForm do
    begin
      sManualDir   := IncludeTrailingPathDelimiter(IF_file.ReadString('Manual', 'ManualDir', DEFAULTMANUALDIR));

      ClientHeight := IF_file.ReadInteger('Manual', 'ManualHeight', DEFAULTMANHEIGHT);
      ClientWidth  := IF_file.ReadInteger('Manual', 'ManualWidth',  DEFAULTMANWIDTH);

      im_page_top.Left   := 0;
      im_page_top.Top    := 0;
      im_page_top.Height := ClientHeight;
      im_page_top.Width  := ClientWidth;

      sh_page_top.Left   := 0;
      sh_page_top.Top    := 0;
      sh_page_top.Height := ClientHeight;
      sh_page_top.Width  := ClientWidth;
    end;

  // instructions manual skin initialization
  for i := 1 to MANUALPAGES do
    begin
      sFilename := IntToStr(i) + '.bmp';

      if FileExists(sManualDir + sFilename) then
          // loads picture into the image
          aIm_pages[i].Picture.LoadFromFile(sManualDir + sFilename)
        else
          // image expected but not found -> creates a coloured square as a placeholder
          begin
            aIm_pages[i].Picture.Bitmap.Create;
            aIm_pages[i].Picture.Bitmap.Width  := 10;
            aIm_pages[i].Picture.Bitmap.Height := 10;
            aIm_pages[i].Picture.Bitmap.Canvas.Create;
            aIm_pages[i].Picture.Bitmap.Canvas.Pen.Color := 0;
            aIm_pages[i].Picture.Bitmap.Canvas.Pen.Style := psSolid;
            aIm_pages[i].Picture.Bitmap.Canvas.Brush.Color := $000000ff;
            aIm_pages[i].Picture.Bitmap.Canvas.Rectangle(0, 0, 10, 10);
          end;

      with ManualForm do
        begin
          aIm_pages[i].Height := ClientHeight;
          aIm_pages[i].Width  := ClientWidth;
        end;
    end;

  // close Inifile
  IF_File.Destroy;
end;





end.
