{ **************************************************************************** }
{ *                                                                          * }
{ * TRON (Tomytronic) Simulator                                              * }
{ * by Luca "MADrigal" Antignano, May 2001 (revision: August 2021)           * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * TABLETOP GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:            TImage;
    btn_game_menu_background:     TImage;

    im_backdrop:                  TImage;
    im_backdrop_logo:             TImage;

    // electronic game
    im_background:                TImage;   im_background_screen_top: TImage;
    im_background_screen_on:      TImage;

    btn_power_on:                 TImage;   btn_power_top:            TImage;
    btn_joystick_down:            TImage;   btn_joystick_top:         TImage;
    btn_action_down:              TImage;   btn_action_top:           TImage;

    btn_ctrlr_overlay_up:         TImage;
    btn_ctrlr_overlay_left:       TImage;
    btn_ctrlr_overlay_right:      TImage;
    btn_ctrlr_overlay_down:       TImage;

{ * VFD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (score)
    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage;

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    // left side panel: Tron (body, hand and disc)
    im_tron_body_5: TImage; im_tron_hand_5: TImage; im_tron_disc_5: TImage;
    im_tron_body_4: TImage; im_tron_hand_4: TImage; im_tron_disc_4: TImage;
    im_tron_body_3: TImage; im_tron_hand_3: TImage; im_tron_disc_3: TImage;
    im_tron_body_2: TImage; im_tron_hand_2: TImage; im_tron_disc_2: TImage;
    im_tron_body_1: TImage; im_tron_hand_1: TImage; im_tron_disc_1: TImage;

    // right side panel: Sark (body, hand and disc)
    im_sark_body_4: TImage; im_sark_hand_4: TImage; im_sark_disc_4: TImage;
    im_sark_body_3: TImage; im_sark_hand_3: TImage; im_sark_disc_3: TImage;
    im_sark_body_2: TImage; im_sark_hand_2: TImage; im_sark_disc_2: TImage;
    im_sark_body_1: TImage; im_sark_hand_1: TImage; im_sark_disc_1: TImage;

    // right side panel: MCP (cover, body and face)
    im_mcp_cover_1: TImage; im_mcp_cover_2: TImage; im_mcp_cover_3: TImage; im_mcp_cover_4: TImage;
    im_mcp_body: TImage;
    im_mcp_face_1: TImage; im_mcp_face_2: TImage; im_mcp_face_3: TImage; im_mcp_face_4: TImage; im_mcp_face_5: TImage;

    // light cycles (horizontal)
                             im_cycle_01_05h: TImage; im_cycle_02_05h: TImage; im_cycle_03_05h: TImage; im_cycle_04_05h: TImage; im_cycle_05_05h: TImage; im_cycle_06_05h: TImage; im_cycle_07_05h: TImage;
                             im_cycle_01_04h: TImage; im_cycle_02_04h: TImage; im_cycle_03_04h: TImage; im_cycle_04_04h: TImage; im_cycle_05_04h: TImage; im_cycle_06_04h: TImage; im_cycle_07_04h: TImage;
                             im_cycle_01_03h: TImage; im_cycle_02_03h: TImage; im_cycle_03_03h: TImage; im_cycle_04_03h: TImage; im_cycle_05_03h: TImage; im_cycle_06_03h: TImage; im_cycle_07_03h: TImage;
                             im_cycle_01_02h: TImage; im_cycle_02_02h: TImage; im_cycle_03_02h: TImage; im_cycle_04_02h: TImage; im_cycle_05_02h: TImage; im_cycle_06_02h: TImage; im_cycle_07_02h: TImage;
    im_cycle_00_01h: TImage; im_cycle_01_01h: TImage; im_cycle_02_01h: TImage; im_cycle_03_01h: TImage; im_cycle_04_01h: TImage; im_cycle_05_01h: TImage; im_cycle_06_01h: TImage; im_cycle_07_01h: TImage; im_cycle_08_01h: TImage;

    // light cycles (vertical)
                             im_cycle_01_05v: TImage; im_cycle_02_05v: TImage; im_cycle_03_05v: TImage; im_cycle_04_05v: TImage; im_cycle_05_05v: TImage; im_cycle_06_05v: TImage; im_cycle_07_05v: TImage;
                             im_cycle_01_04v: TImage; im_cycle_02_04v: TImage; im_cycle_03_04v: TImage; im_cycle_04_04v: TImage; im_cycle_05_04v: TImage; im_cycle_06_04v: TImage; im_cycle_07_04v: TImage;
                             im_cycle_01_03v: TImage; im_cycle_02_03v: TImage; im_cycle_03_03v: TImage; im_cycle_04_03v: TImage; im_cycle_05_03v: TImage; im_cycle_06_03v: TImage; im_cycle_07_03v: TImage;
                             im_cycle_01_02v: TImage; im_cycle_02_02v: TImage; im_cycle_03_02v: TImage; im_cycle_04_02v: TImage; im_cycle_05_02v: TImage; im_cycle_06_02v: TImage; im_cycle_07_02v: TImage;
                             im_cycle_01_01v: TImage; im_cycle_02_01v: TImage; im_cycle_03_01v: TImage; im_cycle_04_01v: TImage; im_cycle_05_01v: TImage; im_cycle_06_01v: TImage; im_cycle_07_01v: TImage;

    // discs (horizontal)
                             im_disc_01_05h: TImage;  im_disc_02_05h: TImage;  im_disc_03_05h: TImage;  im_disc_04_05h: TImage;  im_disc_05_05h: TImage;  im_disc_06_05h: TImage;  im_disc_07_05h: TImage;
                             im_disc_01_04h: TImage;  im_disc_02_04h: TImage;  im_disc_03_04h: TImage;  im_disc_04_04h: TImage;  im_disc_05_04h: TImage;  im_disc_06_04h: TImage;  im_disc_07_04h: TImage;
                             im_disc_01_03h: TImage;  im_disc_02_03h: TImage;  im_disc_03_03h: TImage;  im_disc_04_03h: TImage;  im_disc_05_03h: TImage;  im_disc_06_03h: TImage;  im_disc_07_03h: TImage;
                             im_disc_01_02h: TImage;  im_disc_02_02h: TImage;  im_disc_03_02h: TImage;  im_disc_04_02h: TImage;  im_disc_05_02h: TImage;  im_disc_06_02h: TImage;  im_disc_07_02h: TImage;  im_disc_08_02h: TImage;
                             im_disc_01_01h: TImage;  im_disc_02_01h: TImage;  im_disc_03_01h: TImage;  im_disc_04_01h: TImage;  im_disc_05_01h: TImage;  im_disc_06_01h: TImage;  im_disc_07_01h: TImage;

    // discs (vertical)
                             im_disc_01_05v: TImage;  im_disc_02_05v: TImage;  im_disc_03_05v: TImage;  im_disc_04_05v: TImage;  im_disc_05_05v: TImage;  im_disc_06_05v: TImage;  im_disc_07_05v: TImage;
                             im_disc_01_04v: TImage;  im_disc_02_04v: TImage;  im_disc_03_04v: TImage;  im_disc_04_04v: TImage;  im_disc_05_04v: TImage;  im_disc_06_04v: TImage;  im_disc_07_04v: TImage;
                             im_disc_01_03v: TImage;  im_disc_02_03v: TImage;  im_disc_03_03v: TImage;  im_disc_04_03v: TImage;  im_disc_05_03v: TImage;  im_disc_06_03v: TImage;  im_disc_07_03v: TImage;
                             im_disc_01_02v: TImage;  im_disc_02_02v: TImage;  im_disc_03_02v: TImage;  im_disc_04_02v: TImage;  im_disc_05_02v: TImage;  im_disc_06_02v: TImage;  im_disc_07_02v: TImage;
                             im_disc_01_01v: TImage;  im_disc_02_01v: TImage;  im_disc_03_01v: TImage;  im_disc_04_01v: TImage;  im_disc_05_01v: TImage;  im_disc_06_01v: TImage;  im_disc_07_01v: TImage;

{ * TIMERS: USED IN PHASES 1, 2 AND 3                                        * }
    Timer_Phase_1_2_3_Mcp:       TTimer;

{ * TIMERS: USED IN PHASE 1 ONLY                                             * }
    Timer_Phase_1_Start:         TTimer;
    Timer_Phase_1_Cycle_Tron:    TTimer;
    Timer_Phase_1_Cycle_Sark:    TTimer;
    Timer_Phase_1_Win:           TTimer;
    Timer_Phase_1_Miss:          TTimer;

{ * TIMERS: USED IN PHASE 2 ONLY                                             * }
    Timer_Phase_2_Start:         TTimer;
    Timer_Phase_2_Tron_Move:     TTimer;
    Timer_Phase_2_Sark_Move:     TTimer;
    Timer_Phase_2_Sark_Throw:    TTimer;
    Timer_Phase_2_Tron_Disc:     TTimer;
    Timer_Phase_2_Sark_Disc:     TTimer;
    Timer_Phase_2_Win:           TTimer;
    Timer_Phase_2_Miss_Disc:     TTimer;
    Timer_Phase_2_Miss_Life:     TTimer;

{ * TIMERS: USED IN PHASE 3 ONLY                                             * }
    Timer_Phase_3_Start:         TTimer;
    Timer_Phase_3_Clock:         TTimer;
    Timer_Phase_3_Tron_Disc:     TTimer;
    Timer_Phase_3_Barrier_Open:  TTimer;
    Timer_Phase_3_Barrier_Close: TTimer;
    Timer_Phase_3_Barrier_Move:  TTimer;
    Timer_Phase_3_Win:           TTimer;
    Timer_Phase_3_Miss:          TTimer;

{ * TIMERS: OTHERS                                                           * }
    Timer_Game_Over:             TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
  {}pm_12_controller:   TMenuItem;
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                       (Sender: TObject);
    procedure FormActivate                     (Sender: TObject);
    procedure FormClose                        (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Display_Ctrl_Overlay;

    procedure Vid_Display_All_Sprites          (b: boolean);
    procedure Vid_Display_All_Cycles           (b: boolean);
    procedure Vid_Display_All_Discs            (b: boolean);
    procedure Vid_Display_All_Tron             (b: boolean);
    procedure Vid_Display_All_Sark             (b: boolean);
    procedure Vid_Display_All_Mcp              (b: boolean);

    procedure Vid_Display_Tron_Lives;
    procedure Vid_Display_Sark_Lives;
    procedure Vid_Display_Sark_Lives_Cycles;
    procedure Vid_Display_Tron_Lives_Discs     (b: boolean);
    procedure Vid_Display_Sark_Lives_Discs     (b: boolean);

    procedure Vid_Display_Sark_Cycle           (b: boolean);
    procedure Vid_Display_Grid;

    procedure Vid_Display_Tron                 (b: boolean);
    procedure Vid_Display_Sark                 (b: boolean);
    procedure Vid_Display_Disc                 (b: boolean);

    procedure Vid_Display_Score                (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                         (c: integer; pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove           (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_power_topMouseDown           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_action_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_action_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_ctrlr_overlay_upMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_upMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseDown  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseUp    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_downMouseDown  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_downMouseUp    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                      (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                        (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_power_down;

    procedure Inp_up_down;                     procedure Inp_up_up;
    procedure Inp_down_down;                   procedure Inp_down_up;
    procedure Inp_left_down;                   procedure Inp_left_up;
    procedure Inp_right_down;                  procedure Inp_right_up;

    procedure Inp_action_down;                 procedure Inp_action_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Stop_All_Timers;
    procedure Gam_Checks_Bonus_Life;
    procedure Gam_Phase_1_Clear_Grid;
    procedure Gam_Phase_1_Turn_Tron_Clockwise;
    procedure Gam_Phase_1_Turn_Tron_Counterclockwise;
    procedure Gam_Phase_1_Move_Sark            (p: integer);
    procedure Gam_Phase_1_Turn_Sark            (p: integer);
    procedure Gam_Phase_1_Move_Grid            (p: integer);

    function  Check_Cell_Up:                   boolean;
    function  Check_Cell_Right:                boolean;
    function  Check_Cell_Down:                 boolean;
    function  Check_Cell_Left:                 boolean;

{ * PROCEDURES: TIMERS: USED IN PHASES 1, 2 AND 3                            * }
    procedure Timer_Phase_1_2_3_McpTimer       (Sender: TObject);

{ * PROCEDURES: TIMERS: USED IN PHASE 1 ONLY                                 * }
    procedure Timer_Phase_1_StartTimer         (Sender: TObject);
    procedure Timer_Phase_1_Cycle_TronTimer    (Sender: TObject);
    procedure Timer_Phase_1_Cycle_SarkTimer    (Sender: TObject);
    procedure Timer_Phase_1_WinTimer           (Sender: TObject);
    procedure Timer_Phase_1_MissTimer          (Sender: TObject);

{ * PROCEDURES: TIMERS: USED IN PHASE 2 ONLY                                 * }
    procedure Timer_Phase_2_StartTimer         (Sender: TObject);
    procedure Timer_Phase_2_Tron_MoveTimer     (Sender: TObject);
    procedure Timer_Phase_2_Sark_MoveTimer     (Sender: TObject);
    procedure Timer_Phase_2_Sark_ThrowTimer    (Sender: TObject);
    procedure Timer_Phase_2_Tron_DiscTimer     (Sender: TObject);
    procedure Timer_Phase_2_Sark_DiscTimer     (Sender: TObject);

    procedure Timer_Phase_2_WinTimer           (Sender: TObject);
    procedure Timer_Phase_2_Miss_DiscTimer     (Sender: TObject);
    procedure Timer_Phase_2_Miss_LifeTimer     (Sender: TObject);

{ * PROCEDURES: TIMERS: USED IN PHASE 3 ONLY                                 * }
    procedure Timer_Phase_3_StartTimer         (Sender: TObject);
    procedure Timer_Phase_3_ClockTimer         (Sender: TObject);
    procedure Timer_Phase_3_Tron_DiscTimer     (Sender: TObject);
    procedure Timer_Phase_3_Barrier_OpenTimer  (Sender: TObject);
    procedure Timer_Phase_3_Barrier_CloseTimer (Sender: TObject);
    procedure Timer_Phase_3_Barrier_MoveTimer  (Sender: TObject);
    procedure Timer_Phase_3_WinTimer           (Sender: TObject);
    procedure Timer_Phase_3_MissTimer          (Sender: TObject);

{ * PROCEDURES: TIMERS: OTHERS                                               * }
    procedure Timer_Game_OverTimer             (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                   (Sender: TObject);
    procedure pm_1_aboutClick                  (Sender: TObject);
    procedure pm_2_readmeClick                 (Sender: TObject);
    procedure pm_3_whatsnewClick               (Sender: TObject);
    procedure pm_4_manualClick                 (Sender: TObject);
    procedure pm_6_configClick                 (Sender: TObject);
    procedure pm_7_keysClick                   (Sender: TObject);
  {}procedure pm_10_fullscreenClick            (Sender: TObject);
  {}procedure pm_11_splashscreenClick          (Sender: TObject);
  {}procedure pm_12_controllerClick            (Sender: TObject);
  {}procedure pm_14_exitClick                  (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick           (Sender: TObject);

    procedure la_menu_itemMouseEnter           (Sender: TObject);
    procedure la_menu_itemMouseLeave           (Sender: TObject);
    procedure la_menu_itemMouseUp              (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu            (b: boolean);
end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Tomy (Tomytronic)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME      = HH2_CONTENT_NAME + ' Simulator';                              // application's name (appears on taskbar and Splash screen)
{}APP_VERSION   = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';              // current version (appears on Splash screen)
{}APP_DATE      = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL     = 'mailto:' + HH2_CONTENT_EMAIL;                                // link to the email address
  URL_PAYPAL    = 'https://www.paypal.me/LucaAntignano';                        // link to the Paypal donation page
  URL_WEBSITE   = 'http://' + HH2_CONTENT_WEBSITE + '/';                        // link to the website

  REG_KEY       = 'Software\MADrigal\Tron50';                                   // registry key

  HINT_CLICK    = 'click or press';
  HINT_PWR_OFF  = 'to turn on the game';
  HINT_PWR_ON   = 'to turn off the game';
  HINT_JOYSTICK = 'activate the controller overlays or use';

{}MENU_ITEMS = 13;
  NUM_KEYS = 6;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_beep,
  pfs_complete,
  pfs_disc,
  pfs_enginelo,
  pfs_enginehi,
  pfs_gameover,
  pfs_miss1,
  pfs_miss2,
  pfs_start,
  pfs_turn,
  pfs_win:                  PFSoundSample;                                      // sound samples

  bSound:                   boolean;                                            // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_digit_pos:      array[1..4]          of TImage;                     // empty sprites to host score digits
  aIm_panel_number:         array[0..9]          of TImage;                     // digits for score display

  aIm_cycle_h,                                                                  // cycles/bars (horizontal)
  aIm_cycle_v,                                                                  // cycles/bars (vertical)
  aIm_disc_h,                                                                   // discs (horizontal)
  aIm_disc_v:               array[0..8, 1..5]    of TImage;                     // discs (vertical)

  aIm_tron_body,                                                                // tron (body)
  aIm_tron_hand,                                                                // tron (hand/arm)
  aIm_tron_disc:            array[1..5]          of TImage;                     // tron (discs on shoulder)

  aIm_sark_body,                                                                // sark (body)
  aIm_sark_hand,                                                                // sark (hand/arm)
  aIm_sark_disc:            array[1..4]          of TImage;                     // sark (discs on shoulder)

  aIm_mcp_cover:            array[1..4]          of TImage;                     // MCP (cover parts)
  aIm_mcp_face:             array[1..5]          of TImage;                     // MCP (face parts)

{ * arrays of mixed types *}
  aKeys:                   array[1..NUM_KEYS]    of record                      // keys management (values)
                                                      int:  integer;
                                                      word: word;
                                                    end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * arrays of numbers *}
  aGrid:                    array[-3..11, -3..9] of integer;                    // 0=empty; 1=vert; 2=horz; 3=both; actual grid is [-1..9,-1..7], the remaining cells are used for Sark's AI routines

  aPhase_1_2_3_mcp,                                                             // speed of MCP animation during phases 1 and 2
  aPhase_1_tron_lo,                                                             // speed of tron's cycle during phase 1 (low gear)
  aPhase_1_tron_hi,                                                             // speed of tron's cycle during phase 1 (high gear)
  aPhase_1_sark,                                                                // speed of sark's cycle during phase 1
  aPhase_2_sark,                                                                // speed of sark during phase 2
  aPhase_3_mcp_open,                                                            // speed of MCP animation during phase 3
  aPhase_3_clock:           array[1..6]          of integer;                    // handles the duration of phase 3

{ * images * }
  ImTempPhase1MissCycle,                                                        // cycle to blink when a miss occurs in phase 1
  ImTempPhase1MissDisc:     TImage;                                             // disc to blink when a miss occurs in phase 1

{ * integers and booleans * }
  iMode,                                                                        // 0=game off; 1-3=phases 1-3; 4=game over
  iStage,                                                                       // skill level (1-6)
  iScore,                                                                       // score (0-15999 then resets)
  iScoreBonusLife,                                                              // 2000, 4000, etc.

  iTronLives,                                                                   // Tron's lives (0-5)
  iSarkLives,                                                                   // Sark's lives (0-5)
  iTronDiscs,                                                                   // Tron's discs (0-4)
  iSarkDiscs,                                                                   // Sark's discs (0-4)

  iTronY,                                                                       // Tron's Y coordinate in phase 2 (1-4)
  iSarkX,                                                                       // Sark's X coordinate in phase 1
  iSarkY,                                                                       // Sark's Y coordinate in phases 1 and 2
  iDiscX,                                                                       // disc X coordinate in phase 2 (1-7) - current
  iDiscY,                                                                       // disc Y coordinate in phase 2 (1-4) - current
  iDiscStartingY,                                                               // disc Y coordinate in phase 2 (1-4) - when it is thrown

  iTronCycleDirection,                                                          // direction of Tron's cycle in phase 1 - current  (1=up; 2=right; 3=down; 4=left)
  iSarkCycleDirection,                                                          // direction of Sark's cycle in phase 1 - current  (1=up; 2=right; 3=down; 4=left)
  iTronCycleTurn                                                                // direction of Tron's cycle in phase 1 - turns to (1=up; 2=right; 3=down; 4=left)
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible
  bDisplayCtrlOverlays,                                                         // true if the controller overlay should be displayed

  bController,                                                                  // enables/disables player's action
  bJoyPressed,                                                                  // direction disc pressed/released
  bActionPressed,                                                               // action button pressed/released

  bTronHasDisc,                                                                 // Tron has disc in phase 2
  bSarkHasDisc,                                                                 // Sark has disc in phase 2
  bMcpBarrierOpen                                                               // MCP barrier is open in phase 3
                         : boolean;

  // Game variables: loops
  iLoopPhase123Mcp,                                                             // used in Timer_Phase_1_2_3_Mcp

  iLoopPhase1Start,                                                             // used in Timer_Phase_1_Start
  iLoopPhase1CycleTron,                                                         // used in Timer_Phase_1_Cycle_Tron (loops 1-5 then increases score by 5 points)
  iLoopPhase1Win,                                                               // used in Timer_Phase_1_Win
  iLoopPhase1Miss,                                                              // used in Timer_Phase_1_Miss

  iLoopPhase2Start,                                                             // used in Timer_Phase_2_Start
  iPhase2TronDisc,                                                              // used in Timer_Phase_2_Tron_Disc
  iPhase2SarkDisc,                                                              // used in Timer_Phase_2_Sark_Disc
  iLoopPhase2Win,                                                               // used in Timer_Phase_2_Win
  iLoopPhase2MissDisc,                                                          // used in Timer_Phase_2_Miss_Disc
  iLoopPhase2MissLife,                                                          // used in Timer_Phase_2_Miss_Life

  iLoopPhase3Start,                                                             // used in Timer_Phase_3_Start
  iPhase3Clock,                                                                 // used in Timer_Phase_3_Clock
  iPhase3TronDisc,                                                              // used in Timer_Phase_3_Tron_Disc
  iLoopPhase3BarrierOpen,                                                       // used in Timer_Phase_3_Barrier_Open
  iLoopPhase3BarrierClose,                                                      // used in Timer_Phase_3_Barrier_Close
  iLoopPhase3BarrierMove,                                                       // used in Timer_Phase_3_Barrier_Move
  iLoopPhase3Win,                                                               // used in Timer_Phase_3_Win
  iLoopPhase3Miss                                                               // used in Timer_Phase_3_Miss
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4;

  // numbers
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  aIm_tron_body[1] := im_tron_body_1; aIm_tron_body[2] := im_tron_body_2; aIm_tron_body[3] := im_tron_body_3; aIm_tron_body[4] := im_tron_body_4; aIm_tron_body[5] := im_tron_body_5;
  aIm_tron_hand[1] := im_tron_hand_1; aIm_tron_hand[2] := im_tron_hand_2; aIm_tron_hand[3] := im_tron_hand_3; aIm_tron_hand[4] := im_tron_hand_4; aIm_tron_hand[5] := im_tron_hand_5;
  aIm_tron_disc[1] := im_tron_disc_1; aIm_tron_disc[2] := im_tron_disc_2; aIm_tron_disc[3] := im_tron_disc_3; aIm_tron_disc[4] := im_tron_disc_4; aIm_tron_disc[5] := im_tron_disc_5;

  aIm_sark_body[1] := im_sark_body_1; aIm_sark_body[2] := im_sark_body_2; aIm_sark_body[3] := im_sark_body_3; aIm_sark_body[4] := im_sark_body_4;
  aIm_sark_hand[1] := im_sark_hand_1; aIm_sark_hand[2] := im_sark_hand_2; aIm_sark_hand[3] := im_sark_hand_3; aIm_sark_hand[4] := im_sark_hand_4;
  aIm_sark_disc[1] := im_sark_disc_1; aIm_sark_disc[2] := im_sark_disc_2; aIm_sark_disc[3] := im_sark_disc_3; aIm_sark_disc[4] := im_sark_disc_4;

  aIm_mcp_cover[1] := im_mcp_cover_1; aIm_mcp_cover[2] := im_mcp_cover_2; aIm_mcp_cover[3] := im_mcp_cover_3; aIm_mcp_cover[4] := im_mcp_cover_4;
  aIm_mcp_face[1] := im_mcp_face_1; aIm_mcp_face[2] := im_mcp_face_2; aIm_mcp_face[3] := im_mcp_face_3; aIm_mcp_face[4] := im_mcp_face_4; aIm_mcp_face[5] := im_mcp_face_5;

                                        aIm_cycle_h[1, 5] := im_cycle_01_05h; aIm_cycle_h[2, 5] := im_cycle_02_05h; aIm_cycle_h[3, 5] := im_cycle_03_05h; aIm_cycle_h[4, 5] := im_cycle_04_05h; aIm_cycle_h[5, 5] := im_cycle_05_05h; aIm_cycle_h[6, 5] := im_cycle_06_05h; aIm_cycle_h[7, 5] := im_cycle_07_05h;
                                        aIm_cycle_h[1, 4] := im_cycle_01_04h; aIm_cycle_h[2, 4] := im_cycle_02_04h; aIm_cycle_h[3, 4] := im_cycle_03_04h; aIm_cycle_h[4, 4] := im_cycle_04_04h; aIm_cycle_h[5, 4] := im_cycle_05_04h; aIm_cycle_h[6, 4] := im_cycle_06_04h; aIm_cycle_h[7, 4] := im_cycle_07_04h;
                                        aIm_cycle_h[1, 3] := im_cycle_01_03h; aIm_cycle_h[2, 3] := im_cycle_02_03h; aIm_cycle_h[3, 3] := im_cycle_03_03h; aIm_cycle_h[4, 3] := im_cycle_04_03h; aIm_cycle_h[5, 3] := im_cycle_05_03h; aIm_cycle_h[6, 3] := im_cycle_06_03h; aIm_cycle_h[7, 3] := im_cycle_07_03h;
                                        aIm_cycle_h[1, 2] := im_cycle_01_02h; aIm_cycle_h[2, 2] := im_cycle_02_02h; aIm_cycle_h[3, 2] := im_cycle_03_02h; aIm_cycle_h[4, 2] := im_cycle_04_02h; aIm_cycle_h[5, 2] := im_cycle_05_02h; aIm_cycle_h[6, 2] := im_cycle_06_02h; aIm_cycle_h[7, 2] := im_cycle_07_02h;
  aIm_cycle_h[0, 1] := im_cycle_00_01h; aIm_cycle_h[1, 1] := im_cycle_01_01h; aIm_cycle_h[2, 1] := im_cycle_02_01h; aIm_cycle_h[3, 1] := im_cycle_03_01h; aIm_cycle_h[4, 1] := im_cycle_04_01h; aIm_cycle_h[5, 1] := im_cycle_05_01h; aIm_cycle_h[6, 1] := im_cycle_06_01h; aIm_cycle_h[7, 1] := im_cycle_07_01h; aIm_cycle_h[8, 1] := im_cycle_08_01h;

                                        aIm_cycle_v[1, 5] := im_cycle_01_05v; aIm_cycle_v[2, 5] := im_cycle_02_05v; aIm_cycle_v[3, 5] := im_cycle_03_05v; aIm_cycle_v[4, 5] := im_cycle_04_05v; aIm_cycle_v[5, 5] := im_cycle_05_05v; aIm_cycle_v[6, 5] := im_cycle_06_05v; aIm_cycle_v[7, 5] := im_cycle_07_05v;
                                        aIm_cycle_v[1, 4] := im_cycle_01_04v; aIm_cycle_v[2, 4] := im_cycle_02_04v; aIm_cycle_v[3, 4] := im_cycle_03_04v; aIm_cycle_v[4, 4] := im_cycle_04_04v; aIm_cycle_v[5, 4] := im_cycle_05_04v; aIm_cycle_v[6, 4] := im_cycle_06_04v; aIm_cycle_v[7, 4] := im_cycle_07_04v;
                                        aIm_cycle_v[1, 3] := im_cycle_01_03v; aIm_cycle_v[2, 3] := im_cycle_02_03v; aIm_cycle_v[3, 3] := im_cycle_03_03v; aIm_cycle_v[4, 3] := im_cycle_04_03v; aIm_cycle_v[5, 3] := im_cycle_05_03v; aIm_cycle_v[6, 3] := im_cycle_06_03v; aIm_cycle_v[7, 3] := im_cycle_07_03v;
                                        aIm_cycle_v[1, 2] := im_cycle_01_02v; aIm_cycle_v[2, 2] := im_cycle_02_02v; aIm_cycle_v[3, 2] := im_cycle_03_02v; aIm_cycle_v[4, 2] := im_cycle_04_02v; aIm_cycle_v[5, 2] := im_cycle_05_02v; aIm_cycle_v[6, 2] := im_cycle_06_02v; aIm_cycle_v[7, 2] := im_cycle_07_02v;
                                        aIm_cycle_v[1, 1] := im_cycle_01_01v; aIm_cycle_v[2, 1] := im_cycle_02_01v; aIm_cycle_v[3, 1] := im_cycle_03_01v; aIm_cycle_v[4, 1] := im_cycle_04_01v; aIm_cycle_v[5, 1] := im_cycle_05_01v; aIm_cycle_v[6, 1] := im_cycle_06_01v; aIm_cycle_v[7, 1] := im_cycle_07_01v;

                                        aIm_disc_h[1, 5] := im_disc_01_05h;   aIm_disc_h[2, 5] := im_disc_02_05h;   aIm_disc_h[3, 5] := im_disc_03_05h;   aIm_disc_h[4, 5] := im_disc_04_05h;   aIm_disc_h[5, 5] := im_disc_05_05h;   aIm_disc_h[6, 5] := im_disc_06_05h;   aIm_disc_h[7, 5] := im_disc_07_05h;
                                        aIm_disc_h[1, 4] := im_disc_01_04h;   aIm_disc_h[2, 4] := im_disc_02_04h;   aIm_disc_h[3, 4] := im_disc_03_04h;   aIm_disc_h[4, 4] := im_disc_04_04h;   aIm_disc_h[5, 4] := im_disc_05_04h;   aIm_disc_h[6, 4] := im_disc_06_04h;   aIm_disc_h[7, 4] := im_disc_07_04h;
                                        aIm_disc_h[1, 3] := im_disc_01_03h;   aIm_disc_h[2, 3] := im_disc_02_03h;   aIm_disc_h[3, 3] := im_disc_03_03h;   aIm_disc_h[4, 3] := im_disc_04_03h;   aIm_disc_h[5, 3] := im_disc_05_03h;   aIm_disc_h[6, 3] := im_disc_06_03h;   aIm_disc_h[7, 3] := im_disc_07_03h;
                                        aIm_disc_h[1, 2] := im_disc_01_02h;   aIm_disc_h[2, 2] := im_disc_02_02h;   aIm_disc_h[3, 2] := im_disc_03_02h;   aIm_disc_h[4, 2] := im_disc_04_02h;   aIm_disc_h[5, 2] := im_disc_05_02h;   aIm_disc_h[6, 2] := im_disc_06_02h;   aIm_disc_h[7, 2] := im_disc_07_02h;   aIm_disc_h[8, 2] := im_disc_08_02h;
                                        aIm_disc_h[1, 1] := im_disc_01_01h;   aIm_disc_h[2, 1] := im_disc_02_01h;   aIm_disc_h[3, 1] := im_disc_03_01h;   aIm_disc_h[4, 1] := im_disc_04_01h;   aIm_disc_h[5, 1] := im_disc_05_01h;   aIm_disc_h[6, 1] := im_disc_06_01h;   aIm_disc_h[7, 1] := im_disc_07_01h;

                                        aIm_disc_v[1, 5] := im_disc_01_05v;   aIm_disc_v[2, 5] := im_disc_02_05v;   aIm_disc_v[3, 5] := im_disc_03_05v;   aIm_disc_v[4, 5] := im_disc_04_05v;   aIm_disc_v[5, 5] := im_disc_05_05v;   aIm_disc_v[6, 5] := im_disc_06_05v;   aIm_disc_v[7, 5] := im_disc_07_05v;
                                        aIm_disc_v[1, 4] := im_disc_01_04v;   aIm_disc_v[2, 4] := im_disc_02_04v;   aIm_disc_v[3, 4] := im_disc_03_04v;   aIm_disc_v[4, 4] := im_disc_04_04v;   aIm_disc_v[5, 4] := im_disc_05_04v;   aIm_disc_v[6, 4] := im_disc_06_04v;   aIm_disc_v[7, 4] := im_disc_07_04v;
                                        aIm_disc_v[1, 3] := im_disc_01_03v;   aIm_disc_v[2, 3] := im_disc_02_03v;   aIm_disc_v[3, 3] := im_disc_03_03v;   aIm_disc_v[4, 3] := im_disc_04_03v;   aIm_disc_v[5, 3] := im_disc_05_03v;   aIm_disc_v[6, 3] := im_disc_06_03v;   aIm_disc_v[7, 3] := im_disc_07_03v;
                                        aIm_disc_v[1, 2] := im_disc_01_02v;   aIm_disc_v[2, 2] := im_disc_02_02v;   aIm_disc_v[3, 2] := im_disc_03_02v;   aIm_disc_v[4, 2] := im_disc_04_02v;   aIm_disc_v[5, 2] := im_disc_05_02v;   aIm_disc_v[6, 2] := im_disc_06_02v;   aIm_disc_v[7, 2] := im_disc_07_02v;
                                        aIm_disc_v[1, 1] := im_disc_01_01v;   aIm_disc_v[2, 1] := im_disc_02_01v;   aIm_disc_v[3, 1] := im_disc_03_01v;   aIm_disc_v[4, 1] := im_disc_04_01v;   aIm_disc_v[5, 1] := im_disc_05_01v;   aIm_disc_v[6, 1] := im_disc_06_01v;   aIm_disc_v[7, 1] := im_disc_07_01v;

  // integers: difficulty (game timers speed)
  aPhase_1_2_3_mcp[1]  := 230; aPhase_1_2_3_mcp[2]  := 200; aPhase_1_2_3_mcp[3]  := 170; aPhase_1_2_3_mcp[4]  := 140; aPhase_1_2_3_mcp[5]  := 110; aPhase_1_2_3_mcp[6]  :=  80;
  aPhase_1_tron_lo[1]  := 660; aPhase_1_tron_lo[2]  := 560; aPhase_1_tron_lo[3]  := 490; aPhase_1_tron_lo[4]  := 400; aPhase_1_tron_lo[5]  := 350; aPhase_1_tron_lo[6]  := 280;
  aPhase_1_tron_hi[1]  := 480; aPhase_1_tron_hi[2]  := 410; aPhase_1_tron_hi[3]  := 320; aPhase_1_tron_hi[4]  := 270; aPhase_1_tron_hi[5]  := 230; aPhase_1_tron_hi[6]  := 200;
  aPhase_1_sark[1]     := 560; aPhase_1_sark[2]     := 490; aPhase_1_sark[3]     := 380; aPhase_1_sark[4]     := 330; aPhase_1_sark[5]     := 280; aPhase_1_sark[6]     := 230;
  aPhase_2_sark[1]     := 350; aPhase_2_sark[2]     := 280; aPhase_2_sark[3]     := 240; aPhase_2_sark[4]     := 170; aPhase_2_sark[5]     := 130; aPhase_2_sark[6]     :=  90;
  aPhase_3_mcp_open[1] := 180; aPhase_3_mcp_open[2] := 160; aPhase_3_mcp_open[3] := 140; aPhase_3_mcp_open[4] := 120; aPhase_3_mcp_open[5] := 100; aPhase_3_mcp_open[6] :=  80;
  aPhase_3_clock[1]    := 200; aPhase_3_clock[2]    := 180; aPhase_3_clock[3]    := 160; aPhase_3_clock[4]    := 140; aPhase_3_clock[5]    := 120; aPhase_3_clock[6]    := 100;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "ON/OFF" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 37);                             // "Up"     (default: "UP")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 39);                             // "Left"   (default: "LEFT")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 40);                             // "Right"  (default: "RIGHT")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 38);                             // "Down"   (default: "DOWN")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 42);                             // "Action" (default: "CTRL")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to NUM_KEYS do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_power_top.Hint           := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_PWR_OFF; // "Power off"

  btn_joystick_top.Hint        := HINT_JOYSTICK + ' ' + aKeyHints[aKeys[2].int] + ', ' // "Up"
                                                      + aKeyHints[aKeys[3].int] + ', ' // "Left"
                                                      + aKeyHints[aKeys[4].int] + ', ' // "Right"
                                                      + aKeyHints[aKeys[5].int];       // "Down"

  btn_action_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];          // "Action"

  btn_ctrlr_overlay_up.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int];          // "Up"
  btn_ctrlr_overlay_left.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int];          // "Left"
  btn_ctrlr_overlay_right.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int];          // "Right"
  btn_ctrlr_overlay_down.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];          // "Down"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_9_n2;
{}aMenuItem[9]  := pm_10_fullscreen;
{}aMenuItem[10] := pm_11_splashscreen;
{}aMenuItem[11] := pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // controller overlay setting
{}pm_12_controller.Checked  := iReg.ReadBool('', 'CtrlrOverlay', true);
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

  // controller overlay
  Vid_Display_Ctrl_Overlay;

  // variables init
  DoubleBuffered := true;

  bController    := false;
  bJoyPressed    := false;
  bActionPressed := false;

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_beep);     FSOUND_Sample_Free(pfs_complete);
  FSOUND_Sample_Free(pfs_disc);     FSOUND_Sample_Free(pfs_enginehi);
  FSOUND_Sample_Free(pfs_enginelo); FSOUND_Sample_Free(pfs_gameover);
  FSOUND_Sample_Free(pfs_miss1);    FSOUND_Sample_Free(pfs_miss2);
  FSOUND_Sample_Free(pfs_win);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Display_Ctrl_Overlay;
{ displays or hides the controller overlay images                              }
begin
  btn_ctrlr_overlay_up.Visible    := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_left.Visible  := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_right.Visible := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_down.Visible  := bDisplayCtrlOverlays;

  btn_joystick_top.Visible := (not bDisplayCtrlOverlays);
end;





procedure TForm1.Vid_Display_All_Sprites(b: boolean);
{ displays/hides all sprites, incl. score panel                                }
  var
    i: integer;
begin
  Vid_Display_All_Cycles(b);
  Vid_Display_All_Discs(b);
  Vid_Display_All_Tron(b);
  Vid_Display_All_Sark(b);
  Vid_Display_All_Mcp(b);

  for i := 1 to 4 do
    begin
      aIm_panel_digit_pos[i].Visible := true;

      if b then
        aIm_panel_digit_pos[i].Picture := aIm_panel_number[8].Picture
      else
        aIm_panel_digit_pos[i].Picture := nil;
    end;
end;





procedure TForm1.Vid_Display_All_Cycles(b: boolean);
{ displays/hides all cycles sprites                                            }
  var
    i, j: integer;
begin
  for i := 1 to 7 do
    for j := 1 to 5 do
      begin
        aIm_cycle_h[i, j].Visible := b;
        aIm_cycle_v[i, j].Visible := b;
      end;

  aIm_cycle_h[0, 1].Visible := b;
  aIm_cycle_h[8, 1].Visible := b;
end;





procedure TForm1.Vid_Display_All_Discs(b: boolean);
{ displays/hides all discs sprites                                             }
  var
    i, j: integer;
begin
  for i := 1 to 7 do
    for j := 1 to 5 do
      begin
        aIm_disc_h[i, j].Visible := b;
        aIm_disc_v[i, j].Visible := b;
      end;

  aIm_disc_h[8, 2].Visible := b;
end;





procedure TForm1.Vid_Display_All_Tron(b: boolean);
{ displays/hides all Tron's sprites                                            }
var
  i: integer;
begin
  for i := 1 to 5 do
    begin
      aIm_tron_body[i].Visible := b;
      aIm_tron_hand[i].Visible := b;
      aIm_tron_disc[i].Visible := b;
    end;
end;





procedure TForm1.Vid_Display_All_Sark(b: boolean);
{ displays/hides all Sark's sprites                                            }
  var
    i: integer;
begin
  for i := 1 to 4 do
    begin
      aIm_sark_body[i].Visible := b;
      aIm_sark_hand[i].Visible := b;
      aIm_sark_disc[i].Visible := b;
    end;
end;





procedure TForm1.Vid_Display_All_Mcp(b: boolean);
{ displays/hides all MCP's sprites                                             }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_mcp_cover[i].Visible := b;

  for i := 1 to 5 do
    aIm_mcp_face[i].Visible := b;

  im_mcp_body.Visible := b;
end;





procedure TForm1.Vid_Display_Tron_Lives;
{ displays/hides all Tron's lives sprites                                      }
  var
    i: integer;
begin
  if iTronLives = 0 then
    Vid_Display_All_Tron(false)
  else
    for i := 1 to iTronLives do
      begin
        aIm_tron_body[i].Visible := true;
        aIm_tron_hand[i].Visible := true;
        aIm_tron_disc[i].Visible := true;
      end;
end;





procedure TForm1.Vid_Display_Sark_Lives;
{ displays/hides all Sark's lives sprites in phases 1 and 2                    }
  var
    i: integer;
begin
  if iSarkLives = 0 then
    Vid_Display_All_Sark(false)
  else
    for i := 1 to iSarkLives do
      begin
        aIm_sark_body[i].Visible := true;
        aIm_sark_hand[i].Visible := true;
        aIm_sark_disc[i].Visible := true;
      end;
end;





procedure TForm1.Vid_Display_Sark_Lives_Cycles;
{ displays/hides all Sark's lives/cycles sprites in phase 1                    }
  var
    i: integer;
begin
  if iSarkLives = 1 then
    for i := 5 to 7 do
      begin
        aIm_cycle_h[i, 1].Visible := false;
        aIm_disc_h[i, 1].Visible  := false;
      end
    else
      for i := 7 downto (9 - iSarkLives) do
        begin
          aIm_cycle_h[i, 1].Visible := true;
          aIm_disc_h[i, 1].Visible  := true;
        end;
end;





procedure TForm1.Vid_Display_Tron_Lives_Discs(b: boolean);
{ displays/hides all Tron's lives/discs sprites in phases 2 and 3              }
  var
    i: integer;
begin
  for i := 1 to iTronDiscs do
    aIm_cycle_h[i - 1, 1].Visible := b;
end;





procedure TForm1.Vid_Display_Sark_Lives_Discs(b: boolean);
{ displays/hides all Sark's lives/discs sprites in phase 2                     }
  var
    i: integer;
begin
  for i := 1 to iSarkDiscs do
    aIm_cycle_h[9 - i, 1].Visible := b;
end;





procedure TForm1.Vid_Display_Sark_Cycle(b: boolean);
{ displays/hides Sark's cycle in phase 1                                       }
begin
  if (iSarkX > 0) and (iSarkX < 8) and (iSarkY > 0) and (iSarkY < 6) then
    if Odd(iSarkCycleDirection) then
      begin
        aIm_cycle_v[iSarkX, iSarkY].Visible := b;
        aIm_disc_v[iSarkX,  iSarkY].Visible := b;
      end
    else
      begin
        aIm_cycle_h[iSarkX, iSarkY].Visible := b;
        aIm_disc_h[iSarkX,  iSarkY].Visible := b;
      end;
end;





procedure TForm1.Vid_Display_Grid;
{ displays grid in phase 1                                                     }
  var
    i, j: integer;
begin
  for i := 1 to 7 do
    for j := 1 to 5 do
      begin
        aIm_cycle_v[i, j].Visible := (aGrid[i, j] = 1) or (aGrid[i, j] = 3);    // vert. visible if 1 or 3
        aIm_cycle_h[i, j].Visible := (aGrid[i, j] = 2) or (aGrid[i, j] = 3);    // horz. visible if 2 or 3
      end;
end;





procedure TForm1.Vid_Display_Tron(b: boolean);
{ displays/hides Tron's sprite in phase 2                                      }
begin
  aIm_tron_body[iTronY].Visible := b;

  if bTronHasDisc then
    begin
      aIm_tron_hand[iTronY].Visible := false;
      aIm_tron_disc[iTronY].Visible := b;
    end
  else
    begin
      aIm_tron_hand[iTronY].Visible := b;
      aIm_tron_disc[iTronY].Visible := false;
    end;
end;





procedure TForm1.Vid_Display_Sark(b: boolean);
{ displays/hides Sark's sprite in phase 2                                      }
begin
  aIm_sark_body[iSarkY].Visible := b;

  if bSarkHasDisc then
    begin
      aIm_sark_hand[iSarkY].Visible := false;
      aIm_sark_disc[iSarkY].Visible := b;
    end
  else
    begin
      aIm_sark_hand[iSarkY].Visible := b;
      aIm_sark_disc[iSarkY].Visible := false;
    end;
end;





procedure TForm1.Vid_Display_Disc(b: boolean);
{ displays/hides disc in phases 2  and 3                                       }
begin
  aIm_disc_h[iDiscX, iDiscY].Visible := b;
end;





procedure TForm1.Vid_Display_Score(var p: integer; b: boolean);
{ transforms score into sprites, and displays with/without "bars"              }
  var
    i, ld, lb: integer;
begin
  ld := p; // sets local variable for "score (digits)" to current score (p)
  lb := 0; // sets local variable for "score (bars)"   to zero

  // resets score at 16000
  while ld > 15999 do
    Dec(ld, 16000);

  // counts bars (1 bar = 2000 points)
  while ld > 1999 do
    begin
      Dec(ld, 2000);
      Inc(lb);
    end;

  // displays bars (if b is true)
  if (lb > 0) and b then
    for i := 1 to lb do
      aIm_cycle_h[8 - i, 5].Visible := true;

  // displays digits
  if ld > 999 then aIm_panel_digit_pos[1].picture := aIm_panel_number[ld div 1000 mod 10].picture else aIm_panel_digit_pos[1].picture := nil; // thousands
  if ld >  99 then aIm_panel_digit_pos[2].picture := aIm_panel_number[ld div  100 mod 10].picture else aIm_panel_digit_pos[2].picture := nil; // hundreds
  if ld >   9 then aIm_panel_digit_pos[3].picture := aIm_panel_number[ld div   10 mod 10].picture else aIm_panel_digit_pos[3].picture := nil; // tens
                   aIm_panel_digit_pos[4].picture := aIm_panel_number[ld          mod 10].picture;                                            // units
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(c: integer; pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(c, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_power_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ power switch: CLICK                                                          }
begin
  if Button = mbLeft then
    Inp_power_down;
end;





procedure TForm1.btn_action_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ action button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_action_down;
end;





procedure TForm1.btn_action_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ action button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_action_up;
end;





procedure TForm1.btn_ctrlr_overlay_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ up button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_up_down;
end;





procedure TForm1.btn_ctrlr_overlay_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ up button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_up_up;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ left button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ left button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ right button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ right button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.btn_ctrlr_overlay_downMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ down button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_down_down;
end;





procedure TForm1.btn_ctrlr_overlay_downMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ down button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_down_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_power_down  else                              // Power On/Off

  if Key = aKeys[2].word then Inp_up_down     else                              // Up
  if Key = aKeys[3].word then Inp_left_down   else                              // Left
  if Key = aKeys[4].word then Inp_right_down  else                              // Right
  if Key = aKeys[5].word then Inp_down_down   else                              // Down

  if Key = aKeys[6].word then Inp_action_down;                                  // Action
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[2].word then Inp_up_up     else                                // Up
  if Key = aKeys[3].word then Inp_left_up   else                                // Left
  if Key = aKeys[4].word then Inp_right_up  else                                // Right
  if Key = aKeys[5].word then Inp_down_up   else                                // Down

  if Key = aKeys[6].word then Inp_action_up;                                    // Action
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_power_down;
{ power switch: CLICK                                                          }
begin
  if iMode = 0 then
    // change to Mode 1 (game)
    begin
      btn_power_on.Visible := true;
      im_background_screen_on.Visible := true;
      btn_power_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_PWR_ON; // "click to turn off"
      iMode := 1;

      iLoopPhase1Start := 1;
      Timer_Phase_1_Start.Interval := 320;
      Timer_Phase_1_Start.Enabled := true;
    end
  else
    // change to Mode 0 (off)
    begin
      btn_power_on.Visible := false;
      im_background_screen_on.Visible := false;
      btn_power_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_PWR_OFF; // "click to turn on"
      iMode := 0;

      bController := false;
      Gam_Stop_All_Timers;
      Snd_Silence;
      Vid_Display_All_Sprites(false);
    end;
end;





procedure TForm1.Inp_up_down;
{ up button: PRESS                                                             }
begin
  btn_joystick_down.Visible := true;

  if bController and (iMode = 1) then                                           // phase 1
    case iTronCycleDirection of
      2: iTronCycleTurn := 1;
      4: iTronCycleTurn := 2;
    end; // case..of

  if bController and (not bJoyPressed) and (iMode = 2) and (iTronY < 4) then    // phase 2
    begin
      bJoyPressed := true;

      Vid_Display_Tron(false);
      Inc(iTronY);
      Vid_Display_Tron(true);

      Timer_Phase_2_Tron_Move.Interval := 250;
      Timer_Phase_2_Tron_Move.Enabled := true;
    end;
end;





procedure TForm1.Inp_up_up;
{ up joystick: RELEASE                                                         }
begin
  btn_joystick_down.Visible := false;

  bJoyPressed := false;
end;





procedure TForm1.Inp_left_down;
{ left button: PRESS                                                           }
begin
  btn_joystick_down.Visible := true;

  if bController and (iMode = 1) then                                           // phase 1
    case iTronCycleDirection of
     1: iTronCycleTurn := 7;
     3: iTronCycleTurn := 8;
    end; // case..of
end;





procedure TForm1.Inp_left_up;
{ left joystick: RELEASE                                                       }
begin
  btn_joystick_down.Visible := false;

  bJoyPressed := false;
end;





procedure TForm1.Inp_right_down;
{ right button: PRESS                                                          }
begin
  btn_joystick_down.Visible := true;

  if bController and (iMode = 1) then                                           // phase 1
    case iTronCycleDirection of
      1: iTronCycleTurn := 3;
      3: iTronCycleTurn := 4;
    end; // case..of
end;





procedure TForm1.Inp_right_up;
{ right joystick: RELEASE                                                      }
begin
  btn_joystick_down.Visible := false;

  bJoyPressed := false;
end;





procedure TForm1.Inp_down_down;
{ down button: PRESS                                                           }
begin
  btn_joystick_down.Visible := true;

  if bController and (iMode = 1) then                                           // phase 1
    case iTronCycleDirection of
      2: iTronCycleTurn := 5;
      4: iTronCycleTurn := 6;
    end; // case..of

  if bController and (not bJoyPressed) and (iMode = 2) and (iTronY > 1) then    // phase 2
    begin
      bJoyPressed := true;

      Vid_Display_Tron(false);
      Dec(iTronY);
      Vid_Display_Tron(true);

      Timer_Phase_2_Tron_Move.Interval := 250;
      Timer_Phase_2_Tron_Move.Enabled := true;
   end;

end;





procedure TForm1.Inp_down_up;
{ down joystick: RELEASE                                                       }
begin
  btn_joystick_down.Visible := false;

  bJoyPressed := false;
end;





procedure TForm1.Inp_action_down;
{ action button: PRESS                                                         }
begin
  btn_action_down.Visible := true;

  if not bActionPressed then                                                    // in-game
    begin
      bActionPressed := true;

      if bController and (iMode = 1) then                                         // phase 1
        begin
          Snd_Play(1, pfs_enginehi);
          Timer_Phase_1_Cycle_Tron.Interval := aPhase_1_tron_hi[iStage];
        end;

      if bController and (iMode = 2) and bTronHasDisc then                        // phase 2
        begin
         bTronHasDisc := false;

         iDiscX := 1;
         iDiscY := iTronY;
         iDiscStartingY := iTronY;

         Vid_Display_Disc(true);
         Vid_Display_Tron(true);

         Snd_Play(0, pfs_disc);

         iPhase2TronDisc := 1;
         Timer_Phase_2_Tron_Disc.Interval := 200;
         Timer_Phase_2_Tron_Disc.Enabled := true;
        end;

      if bController and (iMode = 3) and bTronHasDisc then                        // phase 3
        begin
          bTronHasDisc := false;

          iDiscY := 2;
          iDiscX := 1;

          Vid_Display_Disc(true);
          Vid_Display_Tron(True);

          Snd_Play(0, pfs_disc);

          iPhase3TronDisc := 1;
          Timer_Phase_3_Tron_Disc.Interval := 110;
          Timer_Phase_3_Tron_Disc.Enabled := true;
        end;
    end;

  if iMode = 4 then                                                             // game over -> phase 1
    begin
      iMode := 1;

      Vid_Display_All_Sprites(false);

      iLoopPhase1Start := 1;
      Timer_Phase_1_Start.Interval := 320;
      Timer_Phase_1_Start.Enabled := true;
    end;
end;





procedure TForm1.Inp_action_up;
{ action button: RELEASE                                                       }
begin
  btn_action_down.Visible := false;

  if bActionPressed then                                                        // in-game
    begin
      bActionPressed := false;

      if bController and (iMode = 1) then                                         // phase 1
        begin
          Snd_Play(1, pfs_enginelo);
          Timer_Phase_1_Cycle_Tron.Interval := aPhase_1_tron_lo[iStage];
        end;
    end;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all timers at once                                                     }
begin
  // in-game: phases 1, 2 and 3
  Timer_Phase_1_2_3_Mcp.Enabled       := false;

  // in-game: phase 1 only
  Timer_Phase_1_Start.Enabled         := false;
  Timer_Phase_1_Cycle_Tron.Enabled    := false;
  Timer_Phase_1_Cycle_Sark.Enabled    := false;
  Timer_Phase_1_Win.Enabled           := false;
  Timer_Phase_1_Miss.Enabled          := false;

  // in-game: phase 2 only
  Timer_Phase_2_Start.Enabled         := false;
  Timer_Phase_2_Tron_Move.Enabled     := false;
  Timer_Phase_2_Sark_Move.Enabled     := false;
  Timer_Phase_2_Sark_Throw.Enabled    := false;
  Timer_Phase_2_Tron_Disc.Enabled     := false;
  Timer_Phase_2_Sark_Disc.Enabled     := false;
  Timer_Phase_2_Win.Enabled           := false;
  Timer_Phase_2_Miss_Disc.Enabled     := false;
  Timer_Phase_2_Miss_Life.Enabled     := false;

  // in-game: phase 3 only
  Timer_Phase_3_Start.Enabled         := false;
  Timer_Phase_3_Clock.Enabled         := false;
  Timer_Phase_3_Tron_Disc.Enabled     := false;

  Timer_Phase_3_Barrier_Open.Enabled  := false;
  Timer_Phase_3_Barrier_Close.Enabled := false;
  Timer_Phase_3_Barrier_Move.Enabled  := false;

  Timer_Phase_3_Win.Enabled           := false;
  Timer_Phase_3_Miss.Enabled          := false;

  // others
  Timer_Game_Over.Enabled             := false;
end;





procedure TForm1.Gam_Checks_Bonus_Life;
{ checks score for getting a bonus life                                        }
begin
  if iScore >= iScoreBonusLife then
    begin
      if iTronLives < 5 then
        Inc(iTronLives);

      Inc(iScoreBonusLife, 2000);
    end;
end;





procedure TForm1.Gam_Phase_1_Clear_Grid;
{ clears grid data                                                             }
  var
    i, j: integer;
begin
  for i := -3 to 11 do
    for j := -3 to 9 do
      aGrid[i, j] := 0;
end;





procedure TForm1.Gam_Phase_1_Turn_Tron_Clockwise;
{ Tron's cycle turns clockwise                                                 }
begin
  Inc(iTronCycleDirection);

  if iTronCycleDirection > 4 then
    iTronCycleDirection := 1;
end;





procedure TForm1.Gam_Phase_1_Turn_Tron_Counterclockwise;
{ Tron's cycle turns counterclockwise                                          }
begin
  Dec(iTronCycleDirection);

  if iTronCycleDirection < 1 then
    iTronCycleDirection := 4;
end;





procedure TForm1.Gam_Phase_1_Move_Sark(p: integer);
{ Sark's cycle moves 1=up, 2=right, 3=down, 4=left                             }
begin
  Vid_Display_Sark_Cycle(false);

  case p of
    1: begin Inc(iSarkY); Inc(aGrid[iSarkX, iSarkY], 1); end;
    2: begin Inc(iSarkX); Inc(aGrid[iSarkX, iSarkY], 2); end;
    3: begin Dec(iSarkY); Inc(aGrid[iSarkX, iSarkY], 1); end;
    4: begin Dec(iSarkX); Inc(aGrid[iSarkX, iSarkY], 2); end;
  end; // case..of

  Vid_Display_Sark_Cycle(true);
  Vid_Display_Grid;
end;




procedure TForm1.Gam_Phase_1_Turn_Sark(p: integer);
{ Sark's cycle turns 1=up, 2=right, 3=down, 4=left                             }
begin
  Vid_Display_Sark_Cycle(false);

  case p of
    1: begin
         if iSarkCycleDirection = 2 then
           begin // moves right and updates grid
             Inc(iSarkX);
             Inc(aGrid[iSarkX, iSarkY], 1);
           end
         else // updates grid only
           Inc(aGrid[iSarkX,iSarkY], 1);

         iSarkCycleDirection := 1;
       end;
    2: begin
         if iSarkCycleDirection = 1 then
           begin // moves up and updates grid
             Inc(iSarkY);
             Inc(aGrid[iSarkX, iSarkY], 2);
           end
         else // updates grid only
           Inc(aGrid[iSarkX, iSarkY], 2);

         iSarkCycleDirection := 2;
       end;
    3: begin
         if iSarkCycleDirection = 2 then
           begin // moves left and down and updates grid
             Inc(iSarkX);
             Dec(iSarkY);
             Inc(aGrid[iSarkX, iSarkY], 1);
           end
         else
           begin //moves down and updates grid
             Dec(iSarkY);
             Inc(aGrid[iSarkX, iSarkY], 1);
           end;

         iSarkCycleDirection := 3;
       end;
    4: begin
         if iSarkCycleDirection = 1 then
           begin // moves left and up and updates grid
             Dec(iSarkX);
             Inc(iSarkY);
             Inc(aGrid[iSarkX, iSarkY], 2);
           end
         else
           begin // moves left and updates grid
             Dec(iSarkX);
             Inc(aGrid[iSarkX, iSarkY], 2);
           end;

         iSarkCycleDirection := 4;
       end;
  end; // case..of

  Vid_Display_Sark_Cycle(true);
  Vid_Display_Grid;
end;





procedure TForm1.Gam_Phase_1_Move_Grid(p: integer);
{ moves grid 1=up, 2=right, 3=down, 4=left                                     }
  var
    i, j: integer;
begin
  case p of
    1: begin for i := -1 to 9 do begin for j := -1 to 6     do aGrid[i, j] := aGrid[i,     j + 1]; aGrid[i,  7] := 0; end; (* Sark's AI *) for i := -3 to 11 do begin aGrid[i,  9] := aGrid[i, 0]; aGrid[i,  8] := aGrid[i,  -1]; aGrid[i,  -3] := aGrid[i, 6]; aGrid[i,  -2] := aGrid[i, 7]; end; Vid_Display_Sark_Cycle(false); Dec(iSarkY); if iSarkY < -1 then iSarkY :=  7; Vid_Display_Sark_Cycle(true); end;
    2: begin for j := -1 to 7 do begin for i := -1 to 8     do aGrid[i, j] := aGrid[i + 1, j];     aGrid[9,  j] := 0; end; (* Sark's AI *) for j := -3 to 9  do begin aGrid[11, j] := aGrid[0, j]; aGrid[10, j] := aGrid[-1,  j]; aGrid[-3,  j] := aGrid[8, j]; aGrid[-2,  j] := aGrid[9, j]; end; Vid_Display_Sark_Cycle(false); Dec(iSarkX); if iSarkX < -1 then iSarkX :=  9; Vid_Display_Sark_Cycle(true); end;
    3: begin for i := -1 to 9 do begin for j :=  7 downto 0 do aGrid[i, j] := aGrid[i,     j - 1]; aGrid[i, -1] := 0; end; (* Sark's AI *) for i := -3 to 11 do begin aGrid[i,  9] := aGrid[i, 0]; aGrid[i,  8] := aGrid[i,  -1]; aGrid[i,  -3] := aGrid[i, 6]; aGrid[i,  -2] := aGrid[i, 7]; end; Vid_Display_Sark_Cycle(false); Inc(iSarkY); if iSarkY >  7 then iSarkY := -1; Vid_Display_Sark_Cycle(true); end;
    4: begin for j := -1 to 7 do begin for i :=  9 downto 0 do aGrid[i, j] := aGrid[i - 1, j];     aGrid[-1, j] := 0; end; (* Sark's AI *) for j := -3 to 9  do begin aGrid[11, j] := aGrid[0, j]; aGrid[10, j] := aGrid[-1,  j]; aGrid[-3,  j] := aGrid[8, j]; aGrid[-2,  j] := aGrid[9, j]; end; Vid_Display_Sark_Cycle(false); Inc(iSarkX); if iSarkX >  9 then iSarkX := -1; Vid_Display_Sark_Cycle(true); end;
  end; // case..of
end;





function TForm1.Check_Cell_Up: boolean;
{ checks whether cell is empty (up)                                            }
begin
  Check_Cell_Up := true;

  if iSarkCycleDirection = 1 then begin if aGrid[iSarkX - 1, iSarkY + 2] > 1 then Check_Cell_Up := false; if aGrid[iSarkX    , iSarkY + 2] > 0 then Check_Cell_Up := false; end;
  if iSarkCycleDirection = 2 then begin if aGrid[iSarkX,     iSarkY + 1] > 1 then Check_Cell_Up := false; if aGrid[iSarkX + 1, iSarkY + 1] > 0 then Check_Cell_Up := false; end;
  if iSarkCycleDirection = 4 then begin if aGrid[iSarkX - 1, iSarkY + 1] > 1 then Check_Cell_Up := false; if aGrid[iSarkX    , iSarkY + 1] > 0 then Check_Cell_Up := false; end;
end;





function TForm1.Check_Cell_Right: boolean;
{ checks whether cell is empty (rioght)                                        }
begin
  Check_Cell_Right := true;

  if iSarkCycleDirection = 1 then begin if aGrid[iSarkX + 1, iSarkY + 1] > 1 then Check_Cell_Right := false; if (aGrid[iSarkX + 1, iSarkY]     = 1) or (aGrid[iSarkX + 1, iSarkY]     =3) then Check_Cell_Right := false; end;
  if iSarkCycleDirection = 2 then begin if aGrid[iSarkX + 2, iSarkY]     > 0 then Check_Cell_Right := false; if (aGrid[iSarkX + 2, iSarkY - 1] = 1) or (aGrid[iSarkX + 2, iSarkY - 1] =3) then Check_Cell_Right := false; end;
  if iSarkCycleDirection = 3 then begin if aGrid[iSarkX + 1, iSarkY]     > 0 then Check_Cell_Right := false; if (aGrid[iSarkX + 1, iSarkY - 1] = 1) or (aGrid[iSarkX + 1, iSarkY - 1] =3) then Check_Cell_Right := false; end;
end;





function TForm1.Check_Cell_Down: boolean;
{ checks whether cell is empty (down)                                          }
begin
  Check_Cell_Down := true;

  if iSarkCycleDirection = 2 then begin if aGrid[iSarkX + 1, iSarkY - 1] = 2 then Check_Cell_Down := false; if (aGrid[iSarkX + 1, iSarkY - 2] = 1) or (aGrid[iSarkX + 1, iSarkY - 2] = 3) then Check_Cell_Down := false; if aGrid[iSarkX,     iSarkY - 1] = 2 then Check_Cell_Down := false; end;
  if iSarkCycleDirection = 3 then begin if aGrid[iSarkX,     iSarkY - 1] = 2 then Check_Cell_Down := false; if (aGrid[iSarkX,     iSarkY - 2] = 1) or (aGrid[iSarkX,     iSarkY - 2] = 3) then Check_Cell_Down := false; if aGrid[iSarkX - 1, iSarkY - 1] > 1 then Check_Cell_Down := false; end;
  if iSarkCycleDirection = 4 then begin if aGrid[iSarkX,     iSarkY - 1] = 2 then Check_Cell_Down := false; if (aGrid[iSarkX,     iSarkY - 2] = 1) or (aGrid[iSarkX,     iSarkY - 2] = 3) then Check_Cell_Down := false; if aGrid[iSarkX - 1, iSarkY - 1] > 1 then Check_Cell_Down := false; end;
end;





function TForm1.Check_Cell_Left: boolean;
{ checks whether cell is empty (left)                                          }
begin
  Check_Cell_Left := true;

  if iSarkCycleDirection = 1 then begin if  aGrid[iSarkX - 2, iSarkY + 1] > 1 then Check_Cell_Left := false;                                         if aGrid[iSarkX - 1, iSarkY + 1] = 1 then Check_Cell_Left := false; if aGrid[iSarkX - 1, iSarkY] = 1 then Check_Cell_Left := false; end;
  if iSarkCycleDirection = 3 then begin if (aGrid[iSarkX - 1, iSarkY - 1] = 1) or (aGrid[iSarkX - 1, iSarkY - 1] = 3) then Check_Cell_Left := false; if aGrid[iSarkX - 2, iSarkY]     > 1 then Check_Cell_Left := false; if aGrid[iSarkX - 1, iSarkY] = 1 then Check_Cell_Left := false; end;
  if iSarkCycleDirection = 4 then begin if (aGrid[iSarkX - 1, iSarkY - 1] = 1) or (aGrid[iSarkX - 1, iSarkY - 1] = 3) then Check_Cell_Left := false; if aGrid[iSarkX - 2, iSarkY]     > 1 then Check_Cell_Left := false; if aGrid[iSarkX - 1, iSarkY] = 1 then Check_Cell_Left := false; end;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: USED IN PHASES 1, 2 AND 3                            * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Phase_1_2_3_McpTimer(Sender: TObject);
{ MCP animation                                                                }
  var
    i: integer;
begin
  for i := 1 to 5 do
    aIm_mcp_face[i].Visible := iLoopPhase123Mcp <> i;

  Inc(iLoopPhase123Mcp);

  if iLoopPhase123Mcp = 6 then
    iLoopPhase123Mcp := 1;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: USED IN PHASE 1                                      * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Phase_1_StartTimer(Sender: TObject);
{ screen is cleared, Tron's and Sark's lives and cycles counter appear, score  }
{ is set to 0, then game action begins                                         }
  var
    i, j: integer;
begin
  case iLoopPhase1Start of
    1: Snd_Play(0, pfs_beep);
    2: Snd_Play(0, pfs_beep);
    3: Snd_Play(0, pfs_beep);
    4: Snd_Play(0, pfs_beep);
    5: begin Snd_Play(0, pfs_beep);                                                        for i := 1 to 7 do aIm_cycle_v[i, 5].Visible := true; end;
    6: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_v[i, 5].Visible := false; for i := 1 to 7 do aIm_cycle_h[i, 5].Visible := true; end;
    7: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_h[i, 5].Visible := false; for i := 1 to 7 do aIm_cycle_v[i, 4].Visible := true; end;
    8: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_v[i, 4].Visible := false; for i := 1 to 7 do aIm_cycle_h[i, 4].Visible := true; end;
    9: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_h[i, 4].Visible := false; for i := 1 to 7 do aIm_cycle_v[i, 3].Visible := true; end;
   10: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_v[i, 3].Visible := false; for i := 1 to 7 do aIm_cycle_h[i, 3].Visible := true; end;
   11: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_h[i, 3].Visible := false; for i := 1 to 7 do aIm_cycle_v[i, 2].Visible := true; end;
   12: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_v[i, 2].Visible := false; for i := 1 to 7 do aIm_cycle_h[i, 2].Visible := true; end;
   13: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_h[i, 2].Visible := false; for i := 1 to 7 do aIm_cycle_v[i, 1].Visible := true; end;
   14: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_v[i, 1].Visible := false; for i := 1 to 7 do aIm_cycle_h[i, 1].Visible := true; end;
   15: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_h[i, 1].Visible := false; end;
   16: Snd_Play(0, pfs_beep);
   17: Snd_Play(0, pfs_beep);
   18: Snd_Play(0, pfs_beep);
   19: begin Snd_Play(0, pfs_start); Timer_Phase_1_Start.Interval := 2810; end;
   20: begin
         iTronLives := 4;
         iSarkLives := 4;
         iScore := 0;
         iScoreBonusLife := 2000;

         Vid_Display_Tron_Lives;
         Vid_Display_Sark_Lives;
         Vid_Display_Sark_Lives_Cycles;
         Vid_Display_All_Mcp(true);
         Vid_Display_Score(iScore, true);

         Timer_Phase_1_Start.Interval := 2000;
       end;
   21: begin
         Vid_Display_All_Cycles(false);
         Vid_Display_All_Discs(false);
         Vid_Display_All_Tron(false);
         Vid_Display_All_Sark(false);

         Gam_Phase_1_Clear_Grid;
         aGrid[4, 3] := 2;
         aGrid[4, 2] := 2;

         Vid_Display_Grid;
         aIm_disc_h[4, 3].Visible := true;

         iSarkX := 4;
         iSarkY := 2;
         iSarkCycleDirection := 2;
         Vid_Display_Sark_Cycle(true);

         Timer_Phase_1_Start.Interval := 1600;
       end;
   22: begin
         iStage := 1;
         iTronCycleDirection := 2;
         iTronCycleTurn := 0;
         bController := true;

         if bActionPressed then
           begin
             Timer_Phase_1_Cycle_Tron.Interval := aPhase_1_tron_hi[iStage];
             Snd_Play(1, pfs_enginehi);
           end
         else
           begin
             Timer_Phase_1_Cycle_Tron.Interval := aPhase_1_tron_lo[iStage];
             Snd_Play(1, pfs_enginelo);
           end;

         iLoopPhase1CycleTron := 0;
         Timer_Phase_1_Cycle_Tron.Enabled := true;

         Timer_Phase_1_Cycle_Sark.Interval := aPhase_1_sark[iStage];
         Timer_Phase_1_Cycle_Sark.Enabled := true;

         iLoopPhase123Mcp := 1;
         Timer_Phase_1_2_3_Mcp.Interval := aPhase_1_2_3_mcp[iStage];
         Timer_Phase_1_2_3_Mcp.Enabled := true;

         Timer_Phase_1_Start.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase1Start);
end;





procedure TForm1.Timer_Phase_1_Cycle_TronTimer(Sender: TObject);
{ Tron's cycle move, checks for collisions, etc.                               }
  var
    bMiss: boolean;
begin
  // checks if any direction key was pressed -> turn Tron's cycle
  case iTronCycleTurn of
    1: begin Gam_Phase_1_Turn_Tron_Counterclockwise; aIm_disc_h[4, 3].Visible := false; aIm_disc_v[4, 3].Visible := true; if aGrid[5, 3] < 3 then Inc(aGrid[5, 3], 1); Gam_Phase_1_Move_Grid(2);  end;
    2: begin Gam_Phase_1_Turn_Tron_Clockwise;        aIm_disc_h[4, 3].Visible := false; aIm_disc_v[4, 3].Visible := true; if aGrid[4, 3] < 3 then Inc(aGrid[4, 3], 1); end;
    3: begin Gam_Phase_1_Turn_Tron_Clockwise;        aIm_disc_v[4, 3].Visible := false; aIm_disc_h[4, 3].Visible := true; if aGrid[4, 4] < 2 then Inc(aGrid[4, 4], 2); Gam_Phase_1_Move_Grid(1);  end;
    4: begin Gam_Phase_1_Turn_Tron_Counterclockwise; aIm_disc_v[4, 3].Visible := false; aIm_disc_h[4, 3].Visible := true; if aGrid[4, 3] < 2 then Inc(aGrid[4, 3], 2); end;
    5: begin Gam_Phase_1_Turn_Tron_Clockwise;        aIm_disc_h[4, 3].Visible := false; aIm_disc_v[4, 3].Visible := true; if aGrid[5, 2] < 3 then Inc(aGrid[5, 2], 1); Gam_Phase_1_Move_Grid(3); Gam_Phase_1_Move_Grid(2); end;
    6: begin Gam_Phase_1_Turn_Tron_Counterclockwise; aIm_disc_h[4, 3].Visible := false; aIm_disc_v[4, 3].Visible := true; if aGrid[4, 2] < 3 then Inc(aGrid[4, 2], 1); Gam_Phase_1_Move_Grid(3); end;
    7: begin Gam_Phase_1_Turn_Tron_Counterclockwise; aIm_disc_v[4, 3].Visible := false; aIm_disc_h[4, 3].Visible := true; if aGrid[3, 4] < 2 then Inc(aGrid[3, 4], 2); Gam_Phase_1_Move_Grid(1); Gam_Phase_1_Move_Grid(4); end;
    8: begin Gam_Phase_1_Turn_Tron_Clockwise;        aIm_disc_v[4, 3].Visible := false; aIm_disc_h[4, 3].Visible := true; if aGrid[3, 3] < 2 then Inc(aGrid[3, 3], 2); Gam_Phase_1_Move_Grid(4);  end;
  end; // case..of

  // if no direction key was pressed
  if iTronCycleTurn = 0 then
    // moves in the current direction
    case iTronCycleDirection of
      1: begin Gam_Phase_1_Move_Grid(1); if aGrid[4, 3] < 3 then Inc(aGrid[4, 3], 1); end;
      2: begin Gam_Phase_1_Move_Grid(2); if aGrid[4, 3] < 2 then Inc(aGrid[4, 3], 2); end;
      3: begin Gam_Phase_1_Move_Grid(3); if aGrid[4, 3] < 3 then Inc(aGrid[4, 3], 1); end;
      4: begin Gam_Phase_1_Move_Grid(4); if aGrid[4, 3] < 2 then Inc(aGrid[4, 3], 2); end;
    end // case..of
  else
    // Tron's cycle turns, tune is played, and sets "turn" to zero for next timer tick
    begin
      Snd_Play(0, pfs_turn);

      iTronCycleTurn := 0;
    end;

  // loops action (adds 5 points score at every 5 timer ticks)
  Inc(iLoopPhase1CycleTron);

  if iLoopPhase1CycleTron > 5 then
    begin
      Inc(iScore, 5);
      Vid_Display_Score(iScore, false);
      Gam_Checks_Bonus_Life;

      iLoopPhase1CycleTron := 1;
  end;

  // displays updated grid
  Vid_Display_Grid;

  // checks collision between the two cycles
  bMiss := false;

  case iTronCycleDirection of
    1: if (aGrid[3, 4] >= 2) or (aGrid[4, 4] <> 0)                                            then bMiss := true;
    2: if (aGrid[5, 2] =  1) or (aGrid[5, 2] =  3) or (aGrid[5, 3] <> 0)                      then bMiss := true;
    3: if (aGrid[4, 3] =  3) or (aGrid[3, 3] >= 2) or (aGrid[4, 2] =  1) or (aGrid[4, 2] = 3) then bMiss := true;
    4: if (aGrid[4, 3] =  3) or (aGrid[3, 3] >= 2) or (aGrid[4, 2] =  1) or (aGrid[4, 2] = 3) then bMiss := true;
  end; // case..of

  // executes the "miss" routine in case there was collision between the two cycles
  if bMiss then
    begin
      Gam_Stop_All_Timers;
      Snd_Silence;

      bController:=false;

      if Odd(iTronCycleDirection) then
        begin
          ImTempPhase1MissCycle := aIm_cycle_v[4, 3];
          ImTempPhase1MissDisc  := aIm_disc_v[4, 3];
        end
      else
        begin
          ImTempPhase1MissCycle := aIm_cycle_h[4, 3];
          ImTempPhase1MissDisc  := aIm_disc_h[4, 3];
        end;

      iLoopPhase1Miss := 1;
      Timer_Phase_1_Miss.Interval := 100;
      Timer_Phase_1_Miss.Enabled := true;
    end;
end;





procedure TForm1.Timer_Phase_1_Cycle_SarkTimer(Sender: TObject);
{ Sark's AI: attacks or escapes Tron's cycle, depending on their positions     }
  var
    bTurned,
    bMiss: boolean;
begin
  bTurned := false;

  // Sark attacks Tron by turning his cycle in front of Tron's cycle
  case iTronCycleDirection of
    1: case iSarkCycleDirection of                                                                                              // Tron's cycle goes up
         1: begin                                                                                                                 // Sark's cycle goes up
              if (iSarkX < 4) and (iSarkY > 3) and Check_Cell_Right then begin bTurned := true; Gam_Phase_1_Turn_Sark(2); end;      // Sark turns right
              if (iSarkX > 4) and (iSarkY > 3) and Check_Cell_Left  then begin bTurned := true; Gam_Phase_1_Turn_Sark(4); end;      // Sark turns left
             end;
         2: if   (iSarkX > 4) and (iSarkY > 3) and Check_Cell_Down  then begin bTurned := true; Gam_Phase_1_Turn_Sark(3); end;    // Sark's cycle goes right, turns down
         3: begin                                                                                                                 // Sark's cycle goes down
              if (iSarkX < 4) and (iSarkY > 4) and Check_Cell_Right then begin bTurned := true; Gam_Phase_1_Turn_Sark(2); end;      // Sark turns right
              if (iSarkX > 4) and (iSarkY > 4) and Check_Cell_Left  then begin bTurned := true; Gam_Phase_1_Turn_Sark(4); end;      // Sark turns left
             end;
         4: if   (iSarkX < 4) and (iSarkY > 3) and Check_Cell_Down  then begin bTurned := true; Gam_Phase_1_Turn_Sark(3); end;    //Sark's cycle goes left, turns down
       end; // case..of (iSarkCycleDirection)
    2: case iSarkCycleDirection of                                                                                              // Tron's cycle goes right
         1: if   (iSarkX > 4) and (iSarkY > 3) and Check_Cell_Left  then begin bTurned := true; Gam_Phase_1_Turn_Sark(4); end;    // Sark's cycle goes up, turns left
         2: begin                                                                                                                 // Sark's cycle goes right
              if (iSarkX > 4) and (iSarkY < 3) and Check_Cell_Up    then begin bTurned := true; Gam_Phase_1_Turn_Sark(1); end;      // Sark turns up
              if (iSarkX > 4) and (iSarkY > 3) and Check_Cell_Down  then begin bTurned := true; Gam_Phase_1_Turn_Sark(3); end;      // Sark turns down
             end;
         3: if   (iSarkX > 4) and (iSarkY < 3) and Check_Cell_Left  then begin bTurned := true; Gam_Phase_1_Turn_Sark(4); end;    // Sark's cycle goes down, turns left
         4: begin                                                                                                                 // Sark's cycle goes left
              if (iSarkX > 5) and (iSarkY < 3) and Check_Cell_Up    then begin bTurned := true; Gam_Phase_1_Turn_Sark(1); end;      // Sark turns up
              if (iSarkX > 5) and (iSarkY > 3) and Check_Cell_Down  then begin bTurned := true; Gam_Phase_1_Turn_Sark(3); end;      // Sark turns down
             end;
       end; // case..of (iSarkCycleDirection)
    3: case iSarkCycleDirection of                                                                                              // Tron's cycle goes down
         1: begin                                                                                                                 // Sark's cycle goes up
              if (iSarkX < 4) and (iSarkY < 2) and Check_Cell_Right then begin bTurned := true; Gam_Phase_1_Turn_Sark(2); end;      // Sark turns right
              if (iSarkX > 4) and (iSarkY < 2) and Check_Cell_Left  then begin bTurned := true; Gam_Phase_1_Turn_Sark(4); end;      // Sark turns left
             end;
         2: if   (iSarkX > 4) and (iSarkY < 3) and Check_Cell_Up    then begin bTurned := true; Gam_Phase_1_Turn_Sark(1); end;    // Sark's cycle goes right, turns up
         3: begin                                                                                                                 // Sark's cycle goes down
              if (iSarkX < 4) and (iSarkY < 3) and Check_Cell_Right then begin bTurned := true; Gam_Phase_1_Turn_Sark(2); end;      // Sark turns right
              if (iSarkX > 4) and (iSarkY < 3) and Check_Cell_Left  then begin bTurned := true; Gam_Phase_1_Turn_Sark(4); end;      // Sark turns left
            end;
         4: if   (iSarkX < 4) and (iSarkY < 3) and Check_Cell_Up    then begin bTurned := true; Gam_Phase_1_Turn_Sark(1); end;    // Sark's cycle goes left, turns up
       end; // case..of (iSarkCycleDirection)
    4: case iSarkCycleDirection of                                                                                              // Tron's cycle goes down
         1: if   (iSarkX < 4) and (iSarkY > 3) and Check_Cell_Right then begin bTurned := true; Gam_Phase_1_Turn_Sark(2); end;    // Sark's cycle goes up, turns right
         2: begin                                                                                                                 // Sark's cycle goes right
              if (iSarkX < 3) and (iSarkY < 3) and Check_Cell_Up    then begin bTurned := true; Gam_Phase_1_Turn_Sark(1); end;      // Sark turns up
              if (iSarkX < 3) and (iSarkY > 3) and Check_Cell_Down  then begin bTurned := true; Gam_Phase_1_Turn_Sark(3); end;      // Sark turns down
             end;
         3: if   (iSarkX < 4) and (iSarkY < 3) and Check_Cell_Right then begin bTurned := true; Gam_Phase_1_Turn_Sark(2); end;    // Sark's cycle goes down, turns right
         4: begin                                                                                                                 // Sark's cycle goes left
              if (iSarkX < 4) and (iSarkY < 3) and Check_Cell_Up    then begin bTurned := true; Gam_Phase_1_Turn_Sark(1); end;      // Sark turns up
              if (iSarkX < 4) and (iSarkY > 3) and Check_Cell_Down  then begin bTurned := true; Gam_Phase_1_Turn_Sark(3); end;      // Sark turns down
             end;
       end; // case..of (iSarkCycleDirection)
  end; // case..of (iTronCycleDirection)

  // Sark escapes in case he didn't attack before
  if (not bTurned) then
    case iSarkCycleDirection of
      1: if Check_Cell_Up    then Gam_Phase_1_Move_Sark(1) else if Check_Cell_Right then Gam_Phase_1_Turn_Sark(2) else if Check_Cell_Left  then Gam_Phase_1_Turn_Sark(4) else Gam_Phase_1_Move_Sark(1);
      2: if Check_Cell_Right then Gam_Phase_1_Move_Sark(2) else if Check_Cell_Down  then Gam_Phase_1_Turn_Sark(3) else if Check_Cell_Up    then Gam_Phase_1_Turn_Sark(1) else Gam_Phase_1_Move_Sark(2);
      3: if Check_Cell_Down  then Gam_Phase_1_Move_Sark(3) else if Check_Cell_Left  then Gam_Phase_1_Turn_Sark(4) else if Check_Cell_Right then Gam_Phase_1_Turn_Sark(2) else Gam_Phase_1_Move_Sark(3);
      4: if Check_Cell_Left  then Gam_Phase_1_Move_Sark(4) else if Check_Cell_Up    then Gam_Phase_1_Turn_Sark(1) else if Check_Cell_Down  then Gam_Phase_1_Turn_Sark(3) else Gam_Phase_1_Move_Sark(4);
    end; // case..of

  // checks collision between the two cycles
  bMiss := false;

  case iSarkCycleDirection of
    1: begin
         if aGrid[iSarkX,     iSarkY + 1] > 0 then bMiss := true;
         if aGrid[iSarkX - 1, iSarkY + 1] > 1 then bMiss := true;
       end;
    2: begin
         if aGrid[iSarkX + 1, iSarkY]     > 0 then bMiss := true;
         if aGrid[iSarkX + 1, iSarkY - 1] = 1 then bMiss := true;
         if aGrid[iSarkX + 1, iSarkY - 1] = 3 then bMiss := true;
       end;
    3: begin
         if aGrid[iSarkX,     iSarkY]     = 3 then bMiss := true;
         if aGrid[iSarkX,     iSarkY - 1] = 1 then bMiss := true;
         if aGrid[iSarkX,     iSarkY - 1] = 3 then bMiss := true;
         if aGrid[iSarkX - 1, iSarkY]     > 1 then bMiss := true;
       end;
    4: begin
         if aGrid[iSarkX,     iSarkY]     = 3 then bMiss := true;
         if aGrid[iSarkX,     iSarkY - 1] = 1 then bMiss := true;
         if aGrid[iSarkX,     iSarkY - 1] = 3 then bMiss := true;
         if aGrid[iSarkX - 1, iSarkY]     > 1 then bMiss := true;
       end;
  end; // case..of

  // Sark misses in case there was collision between the two cycles
  if bMiss then
    begin
      Gam_Stop_All_Timers;
      Snd_Silence;

      bController := false;

      iLoopPhase1Win := 1;
      Timer_Phase_1_Win.Interval := 100;
      Timer_Phase_1_Win.Enabled := true;
    end;
end;





procedure TForm1.Timer_Phase_1_WinTimer(Sender: TObject);
{ Tron wins one round, goes into phase 2 if Sark has no lives left             }
  var
    i, j: integer;
begin
 if (iLoopPhase1Win > 1) and (iLoopPhase1Win < 22) then
   Vid_Display_Sark_Cycle(Odd(iLoopPhase1Win));

  case iLoopPhase1Win of
    1: begin
         Snd_Play(0, pfs_win);

         Timer_Phase_1_Win.Interval := 1250;
        end;
    2: begin
         Inc(iScore, 100);
         Vid_Display_Score(iScore, false);
         Gam_Checks_Bonus_Life;

         Timer_Phase_1_Win.Interval := 200;
       end;
   22: begin
         Dec(iSarkLives);

         if iSarkLives > 1 then // Sark must have 1 life left to enter phase 2
           // goes back to phase 1
           begin
             Vid_Display_All_Cycles(false);
             Vid_Display_All_Discs(false);
             Vid_Display_Tron_Lives;
             Vid_Display_Sark_Lives;
             Vid_Display_Sark_Lives_Cycles;
             Vid_Display_All_Mcp(true);
             Vid_Display_Score(iScore, true);

             Timer_Phase_1_Win.Interval := 2000;
           end
         else
           // goes into phase 2
           begin
             iLoopPhase2Start := 1;
             Timer_Phase_2_Start.Interval := 320;
             Timer_Phase_2_Start.Enabled := true;

             // stops timer
             Timer_Phase_1_Win.Enabled := false;
           end;
       end;
   23: begin
         // gets ready for a new round in phase 1
         Vid_Display_All_Cycles(false);
         Vid_Display_All_Discs(false);
         Vid_Display_All_Tron(false);
         Vid_Display_All_Sark(false);

         Gam_Phase_1_Clear_Grid;
         aGrid[4, 3] := 2;
         aGrid[4, 2] := 2;

         Vid_Display_Grid;
         aIm_disc_h[4, 3].Visible := true;

         iSarkX := 4;
         iSarkY := 2;
         iSarkCycleDirection := 2;
         Vid_Display_Sark_Cycle(true);

         Timer_Phase_1_Win.Interval := 1600;
       end;
   24: begin
         // gets ready for a new round in phase 1
         iTronCycleDirection := 2;
         iTronCycleTurn := 0;
         bController := true;

         if bActionPressed then
           begin
             Timer_Phase_1_Cycle_Tron.Interval := aPhase_1_tron_hi[iStage];
             Snd_Play(1, pfs_enginehi);
           end
         else
           begin
             Timer_Phase_1_Cycle_Tron.Interval := aPhase_1_tron_lo[iStage];
             Snd_Play(1, pfs_enginelo);
           end;

         iLoopPhase1CycleTron := 0;
         Timer_Phase_1_Cycle_Tron.Enabled := true;

         Timer_Phase_1_Cycle_Sark.Interval := aPhase_1_sark[iStage];
         Timer_Phase_1_Cycle_Sark.Enabled := true;

         iLoopPhase123Mcp := 1;
         Timer_Phase_1_2_3_Mcp.Interval := aPhase_1_2_3_mcp[iStage];
         Timer_Phase_1_2_3_Mcp.Enabled := true;

         Timer_Phase_1_Win.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase1Win);
end;





procedure TForm1.Timer_Phase_1_MissTimer(Sender: TObject);
{ a miss has occured: Tron's cycle blinks, looses one life, goes back to phase }
{ 1 or goes into "game over" if has no lives left                              }
  var
    i, j: integer;
begin
  // cycle blinks during first 20 ticks
  if (iLoopPhase1Miss >= 2) and (iLoopPhase1Miss <= 21) then
    if Odd(iLoopPhase1Miss) then
      begin
        ImTempPhase1MissCycle.Visible := true;
        ImTempPhase1MissDisc.Visible  := true;
      end
    else
      begin
        ImTempPhase1MissCycle.Visible := false;
        ImTempPhase1MissDisc.Visible  := false;
      end;

  case iLoopPhase1Miss of
    1: begin
         Snd_Play(0, pfs_miss1);

         Timer_Phase_1_Miss.Interval := 1250;
       end;
    2: Timer_Phase_1_Miss.Interval := 200;
   22: begin
         Dec(iTronLives);

         if iTronLives > 0 then
           begin
             Vid_Display_All_Cycles(false);
             Vid_Display_All_Discs(false);
             Vid_Display_Tron_Lives;
             Vid_Display_Sark_Lives;
             Vid_Display_Sark_Lives_Cycles;
             Vid_Display_All_Mcp(true);
             Vid_Display_Score(iScore, true);

             Timer_Phase_1_Miss.Interval := 2000;
           end
         else
           begin
             Snd_Play(0, pfs_gameover);

             Timer_Game_Over.Interval := 4000;
             Timer_Game_Over.Enabled := true;

             Timer_Phase_1_Miss.Enabled := false;
           end;
       end;
   23: begin
         Vid_Display_All_Cycles(false);
         Vid_Display_All_Discs(false);
         Vid_Display_All_Tron(false);
         Vid_Display_All_Sark(false);

         Gam_Phase_1_Clear_Grid;
         aGrid[4, 3] := 2;
         aGrid[4, 2] := 2;

         Vid_Display_Grid;
         aIm_disc_h[4, 3].Visible := true;

         iSarkX := 4;
         iSarkY := 2;
         iSarkCycleDirection := 2;
         Vid_Display_Sark_Cycle(true);

         Timer_Phase_1_Miss.Interval := 1600;
       end;
   24: begin
         iTronCycleDirection := 2;
         iTronCycleTurn := 0;
         bController := true;

         if bActionPressed then
           begin
             Timer_Phase_1_Cycle_Tron.Interval := aPhase_1_tron_hi[iStage];
             Snd_Play(1, pfs_enginehi);
           end
         else
           begin
             Timer_Phase_1_Cycle_Tron.Interval := aPhase_1_tron_lo[iStage];
             Snd_Play(1, pfs_enginelo);
           end;

         iLoopPhase1CycleTron := 0;
         Timer_Phase_1_Cycle_Tron.Enabled := true;

         Timer_Phase_1_Cycle_Sark.Interval := aPhase_1_sark[iStage];
         Timer_Phase_1_Cycle_Sark.Enabled := true;

         iLoopPhase123Mcp := 1;
         Timer_Phase_1_2_3_Mcp.Interval := aPhase_1_2_3_mcp[iStage];
         Timer_Phase_1_2_3_Mcp.Enabled := true;

         Timer_Phase_1_Miss.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase1Miss);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: USED IN PHASE 2 ONLY                                 * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Phase_2_StartTimer(Sender: TObject);
{ screen is cleared, Tron's and Sark's lives counter appear, score is          }
{ displayed, then game action begins                                           }
  var
    i: integer;
begin
  case iLoopPhase2Start of
    1: Snd_Play(0, pfs_beep);
    2: Snd_Play(0, pfs_beep);
    3: Snd_Play(0, pfs_beep);
    4: Snd_Play(0, pfs_beep);
    5: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin aIm_disc_v[i,5].Visible:=false; aIm_disc_h[i,5].Visible:=false; aIm_cycle_h[i,5].Visible:=false;                                   aIm_cycle_v[i,5].Visible:=true; end; end;
    6: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin                                                                                                         aIm_cycle_v[i,5].Visible:=false; aIm_cycle_h[i,5].Visible:=true; end; end;
    7: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin aIm_disc_v[i,4].Visible:=false; aIm_disc_h[i,4].Visible:=false; aIm_cycle_h[i,4].Visible:=false; aIm_cycle_h[i,5].Visible:=false; aIm_cycle_v[i,4].Visible:=true; end; end;
    8: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin                                                                                                         aIm_cycle_v[i,4].Visible:=false; aIm_cycle_h[i,4].Visible:=true; end; end;
    9: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin aIm_disc_v[i,3].Visible:=false; aIm_disc_h[i,3].Visible:=false; aIm_cycle_h[i,3].Visible:=false; aIm_cycle_h[i,4].Visible:=false; aIm_cycle_v[i,3].Visible:=true; end; end;
   10: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin                                                                                                         aIm_cycle_v[i,3].Visible:=false; aIm_cycle_h[i,3].Visible:=true; end; end;
   11: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin aIm_disc_v[i,2].Visible:=false; aIm_disc_h[i,2].Visible:=false; aIm_cycle_h[i,2].Visible:=false; aIm_cycle_h[i,3].Visible:=false; aIm_cycle_v[i,2].Visible:=true; end; end;
   12: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin                                                                                                         aIm_cycle_v[i,2].Visible:=false; aIm_cycle_h[i,2].Visible:=true; end; end;
   13: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin aIm_disc_v[i,1].Visible:=false; aIm_disc_h[i,1].Visible:=false; aIm_cycle_h[i,1].Visible:=false; aIm_cycle_h[i,2].Visible:=false; aIm_cycle_v[i,1].Visible:=true; end; end;
   14: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin                                                                                                         aIm_cycle_v[i,1].Visible:=false; aIm_cycle_h[i,1].Visible:=true; end; end;
   15: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do aIm_cycle_h[i,1].Visible:=false; end;
   16: Snd_Play(0, pfs_beep);
   17: Snd_Play(0, pfs_beep);
   18: Snd_Play(0, pfs_beep);
   19: Timer_Phase_2_Start.Interval := 1000;
   20: begin
         Vid_Display_Tron_Lives;
         Vid_Display_Sark_Lives;
         Vid_Display_All_Mcp(true);
         Vid_Display_Score(iScore, true);

         Timer_Phase_2_Start.Interval := 2000;
       end;
   21: begin
         Vid_Display_All_Tron(false);
         Vid_Display_All_Sark(false);
         Vid_Display_All_Cycles(false);

         iTronDiscs := 4;
         Vid_Display_Tron_Lives_Discs(true);

         iSarkDiscs := 4;
         Vid_Display_Sark_Lives_Discs(true);

         iSarkY := 3;
         iTronY := 3;
         bTronHasDisc := false;
         bSarkHasDisc := true;
         Vid_Display_Tron(true);
         Vid_Display_Sark(true);

         Timer_Phase_2_Start.Interval := 1000;
       end;
   22: begin
         iMode := 2;
         bController := true;
         Timer_Phase_2_Sark_Move.Interval := aPhase_2_sark[iStage];
         Timer_Phase_2_Sark_Move.Enabled := true;

         iLoopPhase123Mcp := 1;
         Timer_Phase_1_2_3_Mcp.Interval := aPhase_1_2_3_mcp[iStage];
         Timer_Phase_1_2_3_Mcp.Enabled := true;

         Timer_Phase_2_Start.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase2Start);
end;





procedure TForm1.Timer_Phase_2_Tron_MoveTimer(Sender: TObject);
{ enables controller after moving Tron                                         }
begin
  bJoyPressed := false;

  Timer_Phase_2_Tron_Move.Enabled := false;
end;





procedure TForm1.Timer_Phase_2_Sark_MoveTimer(Sender: TObject);
{ sark moves                                                                   }
begin
  Vid_Display_Sark(false);
  Inc(iSarkY);

  if iSarkY > 4 then
    iSarkY := 1;

  Vid_Display_Sark(true);

  if bSarkHasDisc and (not Timer_Phase_2_Sark_Throw.Enabled) then
    begin
      Timer_Phase_2_Sark_Throw.Interval := aPhase_2_sark[iStage] * (Random(2) + 1);
      Timer_Phase_2_Sark_Throw.Enabled := true;
    end;
end;





procedure TForm1.Timer_Phase_2_Sark_ThrowTimer(Sender: TObject);
{ Sark throws a disc                                                           }
begin
  Snd_Play(0, pfs_disc);

  iDiscX := 7;
  iDiscY := iSarkY;
  iDiscStartingY := iSarkY;
  bSarkHasDisc := false;

  Vid_Display_Disc(true);
  Vid_Display_Sark(true);

  iPhase2SarkDisc := 1;
  Timer_Phase_2_Sark_Disc.Interval := 200;
  Timer_Phase_2_Sark_Disc.Enabled := true;

  Timer_Phase_2_Sark_Move.Enabled := false;
  Timer_Phase_2_Sark_Move.Enabled := true;

  Timer_Phase_2_Sark_Throw.Enabled := false;
end;





procedure TForm1.Timer_Phase_2_Sark_DiscTimer(Sender:TObject);
{ disc moves from Sark to Tron                                                 }
begin
  Vid_Display_Disc(false);

  case iPhase2SarkDisc of
    1: Inc(iDiscY);
    2: case iDiscStartingY of 4: Snd_Play(0, pfs_disc); 3: Inc(iDiscY);           2: Inc(iDiscY);           1: Inc(iDiscY); end;
    3: case iDiscStartingY of 4: Dec(iDiscY);           3: Snd_Play(0, pfs_disc); 2: Inc(iDiscY);           1: Inc(iDiscY); end;
    4: case iDiscStartingY of 4: Dec(iDiscY);           3: Dec(iDiscY);           2: Snd_Play(0, pfs_disc); 1: Inc(iDiscY); end;
    5: case iDiscStartingY of 4: Dec(iDiscY);           3: Dec(iDiscY);           2: Dec(iDiscY);           1: Snd_Play(0, pfs_disc); end;
    6: Dec(iDiscY);
  end; // case..of

  if iPhase2SarkDisc <= 6 then
    begin
      Inc(iPhase2SarkDisc);
      Dec(iDiscX);

      Vid_Display_Disc(true);
    end
  else
    begin
      if iTronY = iDiscY then
        begin
          bTronHasDisc := true;
          Vid_Display_Tron(true);

          Inc(iScore, 5);
          Vid_Display_Score(iScore, false);
          Gam_Checks_Bonus_Life;
        end
      else
        begin
          Gam_Stop_All_Timers;
          Snd_Silence;
          bController := false;
          aIm_tron_disc[iDiscY].Visible := true;

          iLoopPhase2MissDisc := 1;
          Timer_Phase_2_Miss_Disc.Interval := 1;
          Timer_Phase_2_Miss_Disc.Enabled := true;
        end;

      Timer_Phase_2_Sark_Disc.Enabled := false;
    end;
end;





procedure TForm1.Timer_Phase_2_Tron_DiscTimer(Sender: TObject);
{ disc moves from Tron to Sark                                                 }
begin
  Vid_Display_Disc(false);

  case iPhase2TronDisc of
    1: Inc(iDiscY);
    2: case iDiscStartingY of 4: Snd_Play(0, pfs_disc); 3: Inc(iDiscY);           2: Inc(iDiscY);           1: Inc(iDiscY); end;
    3: case iDiscStartingY of 4: Dec(iDiscY);           3: Snd_Play(0, pfs_disc); 2: Inc(iDiscY);           1: Inc(iDiscY); end;
    4: case iDiscStartingY of 4: Dec(iDiscY);           3: Dec(iDiscY);           2: Snd_Play(0, pfs_disc); 1: Inc(iDiscY); end;
    5: case iDiscStartingY of 4: Dec(iDiscY);           3: Dec(iDiscY);           2: Dec(iDiscY);           1: Snd_Play(0, pfs_disc); end;
    6: Dec(iDiscY);
  end; // case..of

  if iPhase2TronDisc <= 6 then
    begin
      Inc(iPhase2TronDisc);
      Inc(iDiscX);

      Vid_Display_Disc(true);
    end
  else
    begin
      if iSarkY = iDiscY then
        begin
          bSarkHasDisc := true;
          Vid_Display_Sark(true);

          Timer_Phase_2_Sark_Throw.Interval := aPhase_2_sark[iStage] * (Random(2) + 1);
          Timer_Phase_2_Sark_Throw.Enabled := true;
        end
      else
        begin
          Gam_Stop_All_Timers;
          Snd_Silence;

          bController := false;

          aIm_sark_disc[iDiscY].Visible := true;

          iLoopPhase2Win := 1;
          Timer_Phase_2_Win.Interval := 1;
          Timer_Phase_2_Win.Enabled := true;
        end;

      Timer_Phase_2_Tron_Disc.Enabled := false;
    end;
end;





procedure TForm1.Timer_Phase_2_WinTimer(Sender: TObject);
{ Sark missed a disc, goes back to action or goes into phase 3                 }
begin
  if iLoopPhase2Win <= 20 then
    begin
      aIm_cycle_h[9 - iSarkDiscs, 1].Visible := Odd(iLoopPhase2Win);

      if Odd(iLoopPhase2Win) then
        Snd_Play(0, pfs_miss2);
    end;

  case iLoopPhase2Win of
    1: Timer_Phase_2_Win.Interval := 35;
   21: begin
         aIm_sark_disc[iDiscY].Visible := false;
         Dec(iSarkDiscs);

         if iSarkDiscs > 0 then
           // score increases, go to case 22 (back to phase 2)
           begin
             bSarkHasDisc := true;
             Vid_Display_Sark(true);

             Inc(iScore, 20);
             Vid_Display_Score(iScore, false);
             Gam_Checks_Bonus_Life;

             Timer_Phase_2_Win.Interval := 1000;
           end
         else
           // timer stops, phase 3 begins
           begin
             iLoopPhase3Start := 1;
             Timer_Phase_3_Start.Interval := 320;
             Timer_Phase_3_Start.Enabled := true;

             Timer_Phase_2_Win.Enabled := false;
           end;
       end;
   22: begin
         // back to phase 2
         Timer_Phase_2_Sark_Move.Enabled := true;

         iLoopPhase123Mcp := 1;
         Timer_Phase_1_2_3_Mcp.Interval := aPhase_1_2_3_mcp[iStage];
         Timer_Phase_1_2_3_Mcp.Enabled := true;

         bController := true;

         Timer_Phase_2_Win.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase2Win);
end;





procedure TForm1.Timer_Phase_2_Miss_DiscTimer(Sender: TObject);
{ a disc was missed, goes back to action or looses a life                      }
begin
  if iLoopPhase2MissDisc <= 20 then
    begin
      aIm_cycle_h[iTronDiscs - 1, 1].Visible := Odd(iLoopPhase2MissDisc);

      if Odd(iLoopPhase2MissDisc) then
        Snd_Play(0, pfs_miss2);
    end;

  case iLoopPhase2MissDisc of
    1: Timer_Phase_2_Miss_Disc.Interval := 35;
   21: begin
         aIm_tron_disc[iDiscY].Visible := false;
         Dec(iTronDiscs);

         if iTronDiscs > 0 then
           // go to case 22 (back to phase 2)
           begin
             bSarkHasDisc := true;
             Vid_Display_Sark(true);

             Timer_Phase_2_Miss_Disc.Interval := 1000;
           end
         else
           // timer stops, lives decrease
           begin
             iLoopPhase2MissLife := 1;
             Timer_Phase_2_Miss_Life.Interval := 100;
             Timer_Phase_2_Miss_Life.Enabled := true;

             Timer_Phase_2_Miss_Disc.Enabled := false;
           end;
       end;
   22: begin
         // back to phase 2
         Timer_Phase_2_Sark_Move.Enabled := true;

         iLoopPhase123Mcp := 1;
         Timer_Phase_1_2_3_Mcp.Interval := aPhase_1_2_3_mcp[iStage];
         Timer_Phase_1_2_3_Mcp.Enabled := true;

         bController := true;

         Timer_Phase_2_Miss_Disc.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase2MissDisc);
end;





procedure TForm1.Timer_Phase_2_Miss_LifeTimer(Sender: TObject);
{ a miss has occured (all discs lost), decreases lives and eventually goes     }
{ into "game over"                                                             }
begin
  case iLoopPhase2MissLife of
    1: begin
         Snd_Play(0, pfs_miss1);

         Timer_Phase_2_Miss_Life.Interval := 1600;
       end;
    2: begin
         Dec(iTronLives);

         if iTronLives > 0 then
           begin
             Vid_Display_All_Cycles(false);
             aIm_cycle_h[0, 1].Visible := false;
             aIm_cycle_h[8, 1].Visible := false;
             Vid_Display_All_Tron(false);
             Vid_Display_All_Sark(false);
             Vid_Display_Tron_Lives;
             Vid_Display_Sark_Lives;
             Vid_Display_All_Mcp(True);

             Timer_Phase_2_Miss_Life.Interval := 2000;
           end
         else
           begin
             Snd_Play(0, pfs_gameover);

             Timer_Game_Over.Interval := 4000;
             Timer_Game_Over.Enabled := true;

             Timer_Phase_2_Miss_Life.Enabled := false;
           end;
       end;
    3: begin
         Vid_Display_All_Tron(false);
         Vid_Display_All_Sark(false);

         iTronDiscs := 4;
         Vid_Display_Tron_Lives_Discs(true);

         iSarkDiscs := 4;
         Vid_Display_Sark_Lives_Discs(true);

         iSarkY := 3;
         iTronY := 3;
         bTronHasDisc := false;
         bSarkHasDisc := true;
         Vid_Display_Tron(true);
         Vid_Display_Sark(true);

         Timer_Phase_2_Miss_Life.Interval := 1600;
       end;
    4: begin
         bController := true;

         Timer_Phase_2_Sark_Move.Enabled := true;

         iLoopPhase123Mcp := 1;
         Timer_Phase_1_2_3_Mcp.Interval := aPhase_1_2_3_mcp[iStage];
         Timer_Phase_1_2_3_Mcp.Enabled := true;

         Timer_Phase_2_Miss_Life.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase2MissLife);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: USED IN PHASE 3 ONLY                                 * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Phase_3_StartTimer(Sender: TObject);
{ screen is cleared, Tron's counter appears, score is displayed, then game     }
{ action begins                                                                }
  var
    i: integer;
begin
  case iLoopPhase3Start of
    1: begin
         Snd_Play(0, pfs_win);

         Inc(iScore, 220); // 20 points for defeating Sark + 200 bonus points *)
         Vid_Display_Score(iScore, false);
         Gam_Checks_Bonus_Life;

         Timer_Phase_3_Start.Interval := 750;
       end;
    2: begin
         Vid_Display_Sark(false);

         Timer_Phase_3_Start.Interval := 500;
       end;
    3: begin
         Snd_Play(0, pfs_beep);

         Timer_Phase_3_Start.Interval := 320;
       end;
    4: Snd_Play(0, pfs_beep);
    5: Snd_Play(0, pfs_beep);
    6: Snd_Play(0, pfs_beep);
    7: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin aIm_disc_v[i, 5].Visible := false; aIm_disc_h[i, 5].Visible := false; aIm_cycle_h[i, 5].Visible := false; aIm_cycle_v[i, 5].Visible := true; end; end;
    8: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin                                                                                                           aIm_cycle_v[i, 5].Visible := false; aIm_cycle_h[i, 5].Visible := true; end; end;
    9: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin aIm_disc_v[i, 4].Visible := false; aIm_disc_h[i, 4].Visible := false; aIm_cycle_h[i, 4].Visible := false; aIm_cycle_h[i, 5].Visible := false; aIm_cycle_v[i, 4].Visible := true; end; end;
   10: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin                                                                                                           aIm_cycle_v[i, 4].Visible := false; aIm_cycle_h[i, 4].Visible := true; end; end;
   11: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin aIm_disc_v[i, 3].Visible := false; aIm_disc_h[i, 3].Visible := false; aIm_cycle_h[i, 3].Visible := false; aIm_cycle_h[i, 4].Visible := false; aIm_cycle_v[i, 3].Visible := true; end; end;
   12: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin                                                                                                           aIm_cycle_v[i, 3].Visible := false; aIm_cycle_h[i, 3].Visible := true; end; end;
   13: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin aIm_disc_v[i, 2].Visible := false; aIm_disc_h[i, 2].Visible := false; aIm_cycle_h[i, 2].Visible := false; aIm_cycle_h[i, 3].Visible := false; aIm_cycle_v[i, 2].Visible := true; end; end;
   14: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin                                                                                                           aIm_cycle_v[i, 2].Visible := false; aIm_cycle_h[i, 2].Visible := true; end; end;
   15: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin aIm_disc_v[i, 1].Visible := false; aIm_disc_h[i, 1].Visible := false; aIm_cycle_h[i, 1].Visible := false; aIm_cycle_h[i, 2].Visible := false; aIm_cycle_v[i, 1].Visible := true; end; end;
   16: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do begin                                                                                                           aIm_cycle_v[i, 1].Visible := false; aIm_cycle_h[i, 1].Visible := true; end; end;
   17: begin Snd_Play(0, pfs_beep); for i:=1 to 7 do aIm_cycle_h[i, 1].Visible := false; end;
   18: Snd_Play(0, pfs_beep);
   19: Snd_Play(0, pfs_beep);
   20: Snd_Play(0, pfs_beep);
   21: Timer_Phase_3_Start.Interval := 1000;
   22: begin
         Vid_Display_All_Tron(false);
         aIm_cycle_h[0, 1].Visible := false;
         Vid_Display_Tron_Lives;
         Vid_Display_All_Mcp(true);
         Vid_Display_Score(iScore, true);

         Timer_Phase_3_Start.Interval := 2000;
       end;
   23: begin
         Vid_Display_All_Tron(false);
         Vid_Display_All_Cycles(false);

         iTronY := 2;
         bTronHasDisc := true;

         Vid_Display_Tron(true);
         Vid_Display_Tron_Lives_Discs(true);

         Timer_Phase_3_Start.Interval := 1000;
       end;
   24: begin
         iMode := 3;
         bMcpBarrierOpen := false;
         bController := true;

         iLoopPhase123Mcp := 1;
         Timer_Phase_1_2_3_Mcp.Interval := aPhase_1_2_3_mcp[iStage];
         Timer_Phase_1_2_3_Mcp.Enabled := true;

         iPhase3Clock := 1;
         Timer_Phase_3_Clock.Interval := aPhase_3_clock[iStage];
         Timer_Phase_3_Clock.Enabled := true;

         Timer_Phase_3_Start.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase3Start);
end;





procedure TForm1.Timer_Phase_3_ClockTimer(Sender: TObject);
{ clock ticks and available time decreases                                     }
begin
  case iPhase3Clock of
        // rightmost disc blinks
    27: aIm_cycle_h[iTronDiscs - 1, 1].Visible := false;
    28: aIm_cycle_h[iTronDiscs - 1, 1].Visible := true;
    29: aIm_cycle_h[iTronDiscs - 1, 1].Visible := false;
    30: aIm_cycle_h[iTronDiscs - 1, 1].Visible := true;
    31: aIm_cycle_h[iTronDiscs - 1, 1].Visible := false;
    32: aIm_cycle_h[iTronDiscs - 1, 1].Visible := true;
    33: aIm_cycle_h[iTronDiscs - 1, 1].Visible := false;
    34: aIm_cycle_h[iTronDiscs - 1, 1].Visible := true;
    35: begin
          // decreases discs
          aIm_cycle_h[iTronDiscs - 1, 1].Visible := false;
          Dec(iTronDiscs);

          // checks if there's still available discs
          if iTronDiscs > 0 then
            // loops action from 1 (ecomes 1 witn next Inc statement)
            iPhase3Clock := 0
          else
            // a miss has occured
            begin
              Gam_Stop_All_Timers;
              Snd_Play(0, pfs_miss1);

              bController := false;

              iLoopPhase3Miss := 1;
              Timer_Phase_3_Miss.Interval := 1600;
              Timer_Phase_3_Miss.Enabled := true;
            end;
        end;
  end; // case..of

  Inc(iPhase3Clock);
end;





procedure TForm1.Timer_Phase_3_Tron_DiscTimer(Sender: TObject);
{ disc moves towards MCP, then hits or misses the MCP core                     }
begin
  if iPhase3TronDisc < 7 then
    begin
      Inc(iPhase3TronDisc);

      Vid_Display_Disc(false);
      Inc(iDiscX);
      Vid_Display_Disc(true);
    end
  else
    begin
      Timer_Phase_3_Barrier_Move.Enabled := false;
      Timer_Phase_3_Clock.Enabled := false;

      // checks if barrier is open
      if bMcpBarrierOpen then
        if aIm_mcp_cover[3].Visible then
          // closes barrier
          begin
            Snd_Play(0, pfs_disc);

            Vid_Display_Disc(false);
            im_sark_disc_2.Visible := true;

            iLoopPhase3BarrierClose := 1;
            Timer_Phase_3_Barrier_Close.Interval := 110;
            Timer_Phase_3_Barrier_Close.Enabled := true;
          end
        else
          // disc entered barrier
          begin
            Snd_Play(0, pfs_complete);

            Timer_Phase_1_2_3_Mcp.Enabled := false;
            Timer_Phase_3_Clock.Enabled := false;

            Vid_Display_Disc(false);
            aIm_disc_h[8, 2].Visible := true;

            iLoopPhase3Win := 1;
            Timer_Phase_3_Win.Interval := 3500;
            Timer_Phase_3_Win.Enabled := true;
          end
      else
        // opens barrier
        begin
          Snd_Play(0, pfs_disc);

          Vid_Display_Disc(false);
          im_sark_disc_2.Visible := true;
          aIm_mcp_cover[3].Visible := false;

          iLoopPhase3BarrierOpen := 1;
          Timer_Phase_3_Barrier_Open.Interval := 110;
          Timer_Phase_3_Barrier_Open.Enabled := true;
        end;

      Timer_Phase_3_Tron_Disc.Enabled := false;
    end;
end;





procedure TForm1.Timer_Phase_3_Barrier_OpenTimer(Sender: TObject);
{ Tron's disc bounces, and MCP barrier is opened                               }
begin
  case iLoopPhase3BarrierOpen of
    1: begin aIm_sark_disc[2].Visible := false; aIm_disc_h[7, 3].Visible := true; end;
    2: begin aIm_disc_h[7, 3].Visible := false; aIm_disc_v[7, 3].Visible := true; end;
    3: begin aIm_disc_v[7, 3].Visible := false; aIm_disc_h[6, 4].Visible := true; end;
    4: begin aIm_disc_h[6, 4].Visible := false; aIm_disc_v[6, 4].Visible := true; end;
    5: begin aIm_disc_v[6, 4].Visible := false; aIm_disc_h[5, 5].Visible := true; end;
    6: begin aIm_disc_h[5, 5].Visible := false; aIm_disc_v[5, 5].Visible := true; end;
    7: begin
         bMcpBarrierOpen := true;
         bTronHasDisc := true;

         Vid_Display_Tron(true);
         aIm_disc_v[5, 5].Visible := false;

         iLoopPhase3BarrierMove := 3;
         Timer_Phase_3_Barrier_Move.Interval := aPhase_3_mcp_open[iStage];
         Timer_Phase_3_Barrier_Move.Enabled := true;

         Timer_Phase_3_Clock.Interval := aPhase_3_clock[iStage];
         Timer_Phase_3_Clock.Enabled := true;

         Timer_Phase_3_Barrier_Open.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase3BarrierOpen);
end;





procedure TForm1.Timer_Phase_3_Barrier_CloseTimer(Sender: TObject);
{ Tron's disc bounces, and MCP barrier is closed                               }
  var
    i: integer;
begin
  case iLoopPhase3BarrierClose of
    1: begin aIm_sark_disc[2].Visible := false; aIm_disc_h[7, 3].Visible := true; end;
    2: begin aIm_disc_h[7, 3].Visible := false; aIm_disc_v[7, 3].Visible := true; end;
    3: begin aIm_disc_v[7, 3].Visible := false; aIm_disc_h[6, 4].Visible := true; end;
    4: begin aIm_disc_h[6, 4].Visible := false; aIm_disc_v[6, 4].Visible := true; end;
    5: begin aIm_disc_v[6, 4].Visible := false; aIm_disc_h[5, 5].Visible := true; end;
    6: begin aIm_disc_h[5, 5].Visible := false; aIm_disc_v[5, 5].Visible := true; end;
    7: begin
         bMcpBarrierOpen := false;
         bTronHasDisc := true;

         Vid_Display_Tron(true);
         aIm_disc_v[5, 5].Visible := false;

         for i := 1 to 4 do
           aIm_mcp_cover[i].Visible := true;

         Timer_Phase_3_Clock.Interval := aPhase_3_clock[iStage];
         Timer_Phase_3_Clock.Enabled := true;

         Timer_Phase_3_Barrier_Close.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase3BarrierClose);
end;





procedure TForm1.Timer_Phase_3_Barrier_MoveTimer(Sender: TObject);
{ moves the "hole" of the MCP barrier                                          }
  var i: integer;
begin
  for i := 1 to 4 do
    aIm_mcp_cover[i].Visible := i <> iLoopPhase3BarrierMove;

  Inc(iLoopPhase3BarrierMove);

  if iLoopPhase3BarrierMove > 4 then
    iLoopPhase3BarrierMove := 1;
end;





procedure TForm1.Timer_Phase_3_WinTimer(Sender: TObject);
{ MCP core was hit, adds score, plays tune and goes back to phase 1            }
  var
    i: integer;
begin
  if iLoopPhase3Win <= 19 then
    if Odd(iLoopPhase3Win) then
      begin
        Vid_Display_All_Mcp(true);
        aIm_disc_h[8, 2].Visible := true;
        aIm_mcp_cover[3].Visible := false;
      end
    else
      begin
        Vid_Display_All_Mcp(false);
        aIm_disc_h[8, 2].Visible := true;
        aIm_mcp_cover[1].Visible := true;
        aIm_mcp_cover[2].Visible := true;
        aIm_mcp_cover[4].Visible := true;
      end;

  case iLoopPhase3Win of
    1: Timer_Phase_3_Win.Interval := 200;
   20: begin
         Inc(iScore, 200);
         Inc(iScore, iTronDiscs * 50);
         Vid_Display_Score(iScore, true);
         Gam_Checks_Bonus_Life;

         Vid_Display_All_Tron(false);
         aIm_disc_h[8, 2].Visible := false;
         Vid_Display_Tron_Lives_Discs(false);
         Vid_Display_Tron_Lives;

         Timer_Phase_3_Win.Interval := 2000;
       end;
   21: begin
         Vid_Display_All_Tron(false);
         Vid_Display_Score(iScore, false);
         Vid_Display_All_Mcp(true);
         Snd_Play(0, pfs_beep);

         Timer_Phase_3_Win.Interval := 320;
       end;
   22: Snd_Play(0, pfs_beep);
   23: Snd_Play(0, pfs_beep);
   24: Snd_Play(0, pfs_beep);
   25: begin Snd_Play(0, pfs_beep);                                                        for i := 1 to 7 do aIm_cycle_v[i, 5].Visible := true; end;
   26: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_v[i, 5].Visible := false; for i := 1 to 7 do aIm_cycle_h[i, 5].Visible := true; end;
   27: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_h[i, 5].Visible := false; for i := 1 to 7 do aIm_cycle_v[i, 4].Visible := true; end;
   28: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_v[i, 4].Visible := false; for i := 1 to 7 do aIm_cycle_h[i, 4].Visible := true; end;
   29: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_h[i, 4].Visible := false; for i := 1 to 7 do aIm_cycle_v[i, 3].Visible := true; end;
   30: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_v[i, 3].Visible := false; for i := 1 to 7 do aIm_cycle_h[i, 3].Visible := true; end;
   31: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_h[i, 3].Visible := false; for i := 1 to 7 do aIm_cycle_v[i, 2].Visible := true; end;
   32: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_v[i, 2].Visible := false; for i := 1 to 7 do aIm_cycle_h[i, 2].Visible := true; end;
   33: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_h[i, 2].Visible := false; for i := 1 to 7 do aIm_cycle_v[i, 1].Visible := true; end;
   34: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_v[i, 1].Visible := false; for i := 1 to 7 do aIm_cycle_h[i, 1].Visible := true; end;
   35: begin Snd_Play(0, pfs_beep); for i := 1 to 7 do aIm_cycle_h[i, 1].Visible := false; end;
   36: Snd_Play(0, pfs_beep);
   37: Snd_Play(0, pfs_beep);
   38: Snd_Play(0, pfs_beep);
   39: begin
         Vid_Display_Tron_Lives;

         iSarkLives := 4;
         Vid_Display_Sark_Lives;
         Vid_Display_Sark_Lives_Cycles;

         Vid_Display_Score(iScore, true);

         Timer_Phase_3_Win.Interval := 2000;
       end;
   40: begin
         Vid_Display_All_Cycles(false);
         Vid_Display_All_Discs(false);
         Vid_Display_All_Tron(false);
         Vid_Display_All_Sark(false);

         Gam_Phase_1_Clear_Grid;

         aGrid[4, 3] := 2;
         aGrid[4, 2] := 2;
         Vid_Display_Grid;
         aIm_disc_h[4, 3].Visible := true;

         iSarkX := 4;
         iSarkY := 2;
         iSarkCycleDirection := 2;
         Vid_Display_Sark_Cycle(true);

         Timer_Phase_3_Win.Interval := 1600;
       end;
   41: begin
         Inc(iStage);

         if iStage > 6 then
           iStage := 6;

         iMode := 1;
         iTronCycleDirection := 2;
         iTronCycleTurn := 0;

         if bActionPressed then
           begin
             Timer_Phase_1_Cycle_Tron.Interval := aPhase_1_tron_hi[iStage];
             Snd_Play(1, pfs_enginehi);
           end
         else
           begin
             Timer_Phase_1_Cycle_Tron.Interval := aPhase_1_tron_lo[iStage];
             Snd_Play(1, pfs_enginelo);
           end;

         iLoopPhase1CycleTron := 0;
         Timer_Phase_1_Cycle_Tron.Enabled := true;

         Timer_Phase_1_Cycle_Sark.Interval := aPhase_1_sark[iStage];
         Timer_Phase_1_Cycle_Sark.Enabled := true;

         iLoopPhase123Mcp := 1;
         Timer_Phase_1_2_3_Mcp.Interval := aPhase_1_2_3_mcp[iStage];
         Timer_Phase_1_2_3_Mcp.Enabled := true;

         bController := true;

         Timer_Phase_3_Win.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase3Win);
end;





procedure TForm1.Timer_Phase_3_MissTimer(Sender: TObject);
{ a miss has occured (time elapsed), decreases lives and eventually goes into  }
{ "game over"                                                                  }
begin
  case iLoopPhase3Miss of
    1: begin
         Dec(iTronLives);

         if iTronLives > 0 then
           // game continues
           begin
             Vid_Display_All_Discs(false);
             Vid_Display_All_Tron(false);

             Vid_Display_Tron_Lives;
             Vid_Display_All_Mcp(true);
             Vid_Display_Score(iScore, true);

             Timer_Phase_3_Miss.Interval := 2000;
           end
         else
           // game is over
           begin
           Snd_Play(0, pfs_gameover);

           Timer_Game_Over.Interval := 4000;
           Timer_Game_Over.Enabled := true;

           // stops timer
           Timer_Phase_3_Miss.Enabled := false;
         end;
       end;
    2: begin
         // displays lives counter
         Vid_Display_All_Tron(false);

         bTronHasDisc := true;
         bMcpBarrierOpen := false;
         iTronDiscs := 4;

         Vid_Display_Tron(true);
         Vid_Display_Tron_Lives_Discs(true);

         Timer_Phase_3_Miss.Interval := 1000;
       end;
    3: begin
         // game reprises
         iLoopPhase123Mcp := 1;
         Timer_Phase_1_2_3_Mcp.Interval := aPhase_1_2_3_mcp[iStage];
         Timer_Phase_1_2_3_Mcp.Enabled := true;

         iPhase3Clock := 1;
         Timer_Phase_3_Clock.Interval := aPhase_3_clock[iStage];
         Timer_Phase_3_Clock.Enabled := true;

         bController := true;

         Timer_Phase_3_Miss.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopPhase3Miss);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: OTHERS                                               * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ hides sprites on game screen, displays sarks and MCP, then goes into "game   }
{ over" mode                                                                   }
begin
  Vid_Display_All_Sprites(false);
  Vid_Display_Score(iScore, true);

  Vid_Display_All_Mcp(true);

  // displays Sark if game over occured in phase 1 or 2
  if iMode <> 3 then
    Vid_Display_Sark_Lives;

  iMode := 4;                                                                   // in-game -> game over

  Timer_Game_Over.Enabled := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);
{}pm_12_controller.Checked := bDisplayCtrlOverlays;

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_12_controllerClick(Sender: TObject);
{ displays the "tranlucent" controller overlay on the directional cross        }
begin
{}pm_12_controller.Checked := not pm_12_controller.Checked;
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

{}iReg.WriteBool('', 'CtrlrOverlay', pm_12_controller.Checked);

  Form1.Vid_Display_Ctrl_Overlay;
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Size := -12;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.

